"""Book Sources"""

# Importing necessary modules
import os
import re
import subprocess
import time

import requests
import urllib3.exceptions
from bs4 import BeautifulSoup
from downitall.resources.globalfunctions import Book, Globalfunctions
from selenium.common.exceptions import (
    SessionNotCreatedException,
    TimeoutException,
    WebDriverException,
)
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys

driver = Globalfunctions().webdriver()[0]
bookcli = Globalfunctions().webdriver()[1]


class Booksources:

    ############################################################################
    class Libgen:
        def libgen_search(self, search_term, choice):
            Globalfunctions().directory(name="Books")
            # Declaring function level variables.
            label = "paged"
            index = 1
            # Making search term better for url through regex.
            term = re.sub(r"\W", "+", search_term)
            # Checking weather search is for title or author.
            if choice == 1:
                search_type = "t"
            else:
                search_type = "a"
            # Url to access the website.
            web_url = f"https://libgen.li/index.php?req={term}&columns%5B%5D={search_type}&objects%5B%5D=f&topics%5B%5D=l&topics%5B%5D=f&res=50&filesuns=all"
            # Url to be used for page navigation.
            page_url = f"https://libgen.li/index.php?req={term}&columns%5B%5D={search_type}&objects%5B%5D=f&topics%5B%5D=l&topics%5B%5D=f&res=50&filesuns=all&page="
            # Url to access the base website
            base_url = "https://libgen.li/"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Checking weather the element exists.
                    if Globalfunctions().exists(element=".paginator.fullsize"):
                        html_tag = soup.find("div", class_="paginator fullsize")
                        # Finding the last td tag and getting its text which is the number of last page and converting it into integer.
                        pages = int(html_tag.find_all("td", colspan=None)[-1].getText())
                    else:
                        pages = 1
                    # Finding all the mentioned elements from webpage.
                    table = soup.find("table", id="tablelibgen")
                    # Calling the table function.
                    df = Globalfunctions().visual_tables(table)
                    # Selecting columns that user can see.
                    df_table = df.iloc[:, [0, 6, 7]]
                    # Making table index start from 1.
                    df_table.index += 1
                    # Converting index column into index list.
                    index_list = df_table.index.tolist()
                    name_list = []
                    # Converting name and extention columns into list and looping them together by using zip.
                    for name, extention in zip(
                        df[df.columns[0]].values.tolist(),
                        df[df.columns[7]].values.tolist(),
                    ):
                        name_list.append(f"{name}.{extention}")
                    link_list = []
                    # Finding the links and making link list.
                    html_tag = soup.find_all(
                        "a", attrs={"data-original-title": "libgen"}
                    )
                    for links in html_tag:
                        link_list.append(base_url + links["href"])
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    index_list.append(len(df_table.index.tolist()) + 1)
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    print("\n0. Next Page")
                    print(df_table)
                    print(f"{len(df_table.index.tolist())+1}. Previous Page")
                    # Calling the user choice function.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    index = Globalfunctions().page_navigation(name, pages, index)
                    web_url = url + str(index)
                    if name != "Next Page" and name != "Previous Page":
                        break
                # Sending get request to the website.
                driver.get(url)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements in the webpage.
                td_html = soup.find(
                    "td",
                    attrs={"align": "center", "valign": "top", "bgcolor": "#A9F5BC"},
                )
                html_tag = td_html.find("a")
                url = base_url + html_tag["href"]
                Book().download(name, url)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except requests.exceptions.RequestException as e:
                print(e)
            except WebDriverException as e:
                print(e)
            except (TypeError, urllib3.exceptions.ProtocolError) as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    class Annas_Archive:
        def annas_archive_search(self, search_term, choice):
            Globalfunctions().directory(name="Books")
            label = "paged"
            index = 1
            # Making search term better for url through regex.
            term = re.sub(r"\W", "+", search_term)
            # Checking weather search is for title or author.
            if choice == 1:
                search_type = "title"
            else:
                search_type = "author"
            # Url to access the searching.
            web_url = f"https://annas-archive.org/search?index=&page=1&desc=1&termtype_1={search_type}&termval_1={term}&content=book_nonfiction&content=book_fiction&content=book_unknown&sort="
            # Url to be used for page navigation.
            page_url = f"https://annas-archive.org/search?index=&desc=1&termtype_1={search_type}&termval_1={term}&content=book_nonfiction&content=book_fiction&content=book_unknown&sort=&page="
            # Url to access the base website
            base_url = "https://annas-archive.org"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    time.sleep(2)
                    # Scrolling to bottom to get all the elements.
                    elem = driver.find_element(By.TAG_NAME, "html")
                    elem.send_keys(Keys.END)
                    time.sleep(2)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Checking weather the element exists.
                    if Globalfunctions().exists(
                        element=".isolate.inline-flex.-space-x-px.rounded-md.shadow-sm.text-xs"
                    ):
                        html_tag = soup.find(
                            "nav",
                            class_="isolate inline-flex -space-x-px rounded-md shadow-sm text-xs",
                        )
                        pages = int(
                            html_tag.find_all(
                                "a",
                                class_="custom-a relative inline-flex items-center px-2 py-2 font-semibold text-gray-900 ring-1 ring-inset ring-gray-300 hover:bg-gray-50 focus:z-20 focus:outline-offset-0",
                            )[-1].getText()
                        )
                    else:
                        pages = 1
                    # Finding all the mentioned elements from webpage.
                    h3_tag = soup.find_all(
                        "h3",
                        class_="max-lg:line-clamp-[2] lg:truncate leading-[1.2] lg:leading-[1.35] text-md lg:text-xl font-bold",
                    )
                    # Creating index, name and link list and appending the respective items.
                    index_list = []
                    name_list = []
                    link_list = []
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    for i, name in enumerate(h3_tag, start=1):
                        name_list.append(name.text.strip())
                        index_list.append(i)
                    a_tag = soup.find_all(
                        "a",
                        class_="js-vim-focus h-[110px] custom-a flex items-center relative left-[-10px] w-[calc(100%+20px)] px-2.5 outline-offset-[-2px] outline-2 rounded-[3px] hover:bg-black/6.7 focus:outline",
                    )
                    for links in a_tag:
                        link_list.append(base_url + links["href"])
                    index_list.append(i + 1)
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    # Iterating using zip as it lets two list loop together
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    index = Globalfunctions().page_navigation(name, pages, index)
                    web_url = url + str(index)
                    if name != "Next Page" and name != "Previous Page":
                        break
                # Sending get request to the website.
                driver.get(url)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements in the webpage.
                html_tag = soup.find_all("ul", class_="list-inside mb-4 ml-1")[1]
                link = html_tag.find_all("a", class_="js-download-link")[-1]
                url = base_url + link["href"]
                print(url)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except requests.exceptions.RequestException as e:
                print(e)
            except WebDriverException as e:
                print(e)
            except (TypeError, FileNotFoundError) as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    class Torrent:
        def torrent_search(self, search_term):
            Globalfunctions().directory(name="Books")
            label = "paged"
            index = 1
            term = re.sub(r"\W", "+", search_term)
            # Url to access the searching.
            web_url = (
                f"https://1337x.to/sort-category-search/{term}/Other/seeders/desc/1/"
            )
            page_url = (
                f"https://1337x.to/sort-category-search/{term}/Other/seeders/desc/"
            )
            # Url to access the base website
            base_url = "https://1337x.to"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    if Globalfunctions().exists(element=".pagination"):
                        html_tag = soup.find("div", class_="pagination")
                        if html_tag.find("li", class_="last"):
                            tag = html_tag.find("li", class_="last")
                            pages = int(
                                tag.find("a")["href"].split("desc/")[1].strip("/")
                            )
                        else:
                            pages = 1
                    else:
                        pages = 1
                    # Finding all the mentioned elements from webpage.
                    table = soup.find(
                        "table",
                        class_="table-list table table-responsive table-striped",
                    )
                    df = Globalfunctions().visual_tables(table)
                    df_table = df.iloc[:, [0, 1]]
                    df_table.index += 1
                    index_list = df_table.index.tolist()
                    name_list = df[df.columns[0]].values.tolist()
                    link_list = []
                    td_tag = table.find_all("td", class_="coll-1 name")
                    for a_tag in td_tag:
                        links = a_tag.find("a", text=True)
                        link_list.append(base_url + links["href"])
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    index_list.append(len(df_table.index.tolist()) + 1)
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    print("\n0. Next Page")
                    print(df_table)
                    print(f"{len(df_table.index.tolist())+1}. Previous Page")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    index = Globalfunctions().page_navigation(name, pages, index)
                    web_url = url + str(index) + "/"
                    if name != "Next Page" and name != "Previous Page":
                        break
                # Sending get request to the website.
                driver.get(url)
                # Parsering the response with "BeauitifulSoup".
                book_soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements in the webpage.
                html_tag = book_soup.find("a", id="id1")
                url = html_tag["href"]
                dir = os.getcwd()
                # Calling the aria2c module to download.
                args = [
                    "aria2c",
                    "--file-allocation=none",
                    "--seed-time=0",
                    "-d",
                    dir,
                    url,
                ]
                subprocess.call(args)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except requests.exceptions.RequestException as e:
                print(e)
            except WebDriverException as e:
                print(e)
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    class Rutracker:
        def rutracker(self, choice):
            Globalfunctions().directory(name="Books")
            label = "no_page"
            index = 1
            if choice == 1:
                search_type = "1556"
            else:
                search_type = "610"
            # Url to access the searching.
            web_url = f"https://rutracker-net.translate.goog/forum/viewforum.php?f={search_type}&_x_tr_sl=ru&_x_tr_tl=en&_x_tr_hl=en&_x_tr_pto=wapp"
            # Url to access the base website
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                scroll_amount = 1000
                driver.execute_script(f"window.scrollBy(0, {scroll_amount})")
                time.sleep(5)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                table = soup.find_all("table", class_="forumline forum")[0]
                index_list = []
                name_list = []
                link_list = []
                html_tag = table.find_all("h4", class_="forumlink")
                for i, tag in enumerate(html_tag, start=1):
                    index_list.append(i)
                    name_list.append(tag.text.strip())
                    links = tag.find("a", text=True)
                    link_list.append(links["href"])
                # Iterating using zip as it lets two list loop together
                print("\n")
                for index_name, name in zip(index_list, name_list):
                    # Printing name with index for user to choose.
                    print(f"{index_name}. {name}")
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                web_url = data[0]
                # Sending get request to the website.
                label = "paged"
                page_url = web_url + "&start="
                while True:
                    driver.get(web_url)
                    time.sleep(5)
                    scroll_amount = 1000
                    driver.execute_script(f"window.scrollBy(0, {scroll_amount})")
                    time.sleep(5)
                    scroll_amount = 1000
                    driver.execute_script(f"window.scrollBy(0, {scroll_amount})")
                    time.sleep(5)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    if Globalfunctions().exists(element=".pg"):
                        pages = int(soup.find_all("a", class_="pg")[-2].text.strip())
                    else:
                        pages = 1
                    # Finding all the mentioned elements from webpage.
                    index_list = []
                    name_list = []
                    link_list = []
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    html_tag = soup.find_all("a", class_="torTopic bold tt-text")
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        name_list.append(links.text.strip())
                        link_list.append(links["href"])
                    index_list.append(i + 1)
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    # Iterating using zip as it lets two list loop together
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    index = Globalfunctions().page_navigation(name, pages, index)
                    web_url = url + str(index * 50)
                    if name != "Next Page" and name != "Previous Page":
                        break
                driver.get(url)
                time.sleep(5)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                html_tag = soup.find_all("ul", class_="inlined middot-separated")[0]
                tag = html_tag.find_all("a")[0]
                url = tag["href"]
                dir = os.getcwd()
                # Calling the aria2c module to download.
                args = [
                    "aria2c",
                    "--file-allocation=none",
                    "--seed-time=0",
                    "-d",
                    dir,
                    url,
                ]
                subprocess.call(args)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except requests.exceptions.RequestException as e:
                print(e)
            except WebDriverException as e:
                print(e)
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    class Goldenaudiobooks:
        def goldenaudiobooks(self, search_term):
            Globalfunctions().directory(name="Books")
            label = "paged"
            # Url to access the searching.
            web_url = f"https://goldenaudiobooks.club/?s={search_term}"
            try:
                while True:
                    driver.get(web_url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    index_list = []
                    name_list = []
                    link_list = []
                    img_list = []
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append("")
                    html_tag = soup.find_all("div", class_="post-cover")
                    for i, tag in enumerate(html_tag, start=1):
                        index_list.append(i)
                        for links in tag.find_all("a"):
                            name_list.append(links["title"].strip())
                            link_list.append(links["href"])
                        for img in tag.find_all("img"):
                            img_list.append(img["src"])
                    index_list.append(i + 1)
                    name_list.append("Previous Page")
                    link_list.append("")
                    # Iterating using zip as it lets two list loop together
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if name == "Next Page":
                        if Globalfunctions().exists(element=".nav-previous"):
                            html_tag = soup.find_all("div", class_="nav-previous")[0]
                            web_url = html_tag.find_all("a")[0]["href"]
                        else:
                            pass
                    elif name == "Previous Page":
                        if Globalfunctions().exists(element=".nav-next"):
                            html_tag = soup.find_all("div", class_="nav-next")[0]
                            web_url = html_tag.find_all("a")[0]["href"]
                        else:
                            pass
                    else:
                        break
                driver.get(url)
                book_name = re.sub("[^a-z,0-9.]", "_", name, flags=re.IGNORECASE)
                if not os.path.isdir(book_name):
                    os.mkdir(book_name)
                    os.chdir(book_name)
                else:
                    os.chdir(book_name)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                html_tag = soup.find_all("audio", class_="wp-audio-shortcode")
                for i, links in enumerate(html_tag, start=1):
                    url = links["src"].split("?_")[0]
                    name = f"Chapter {i}"
                    subprocess.call(["wget", "-O", name, url])
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except requests.exceptions.RequestException as e:
                print(e)
            except WebDriverException as e:
                print(e)
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")


############################################################################



"""Importing Booksources from booksources in resource directory"""

from downitall.resources.animesources import Animesources
from downitall.resources.booksources import Booksources
from downitall.resources.globalfunctions import Globalfunctions
from downitall.resources.mangasources import Mangasources
from downitall.resources.musicsources import Musicsources
from downitall.resources.streamsources import Streamsources
from downitall.resources.tvsources import Tvsources



"""Bookcli"""

# Importing packages required for the program.
import os
import subprocess

import questionary

from downitall.resources import (
    Animesources,
    Booksources,
    Mangasources,
    Musicsources,
    Streamsources,
    Tvsources,
)

# Space for functions start from here


############################################################################
# Defining the core function of UI
def anime():
    # Second question to choose the Website.
    search_term = input("Enter the title of the Anime: ")
    select = questionary.select(
        "Select item",
        choices=["TokyoInsider", "Nyaa", "Kayoanime", "Nibl", "Animk", "Reset", "Exit"],
    ).ask()
    if select == "TokyoInsider":
        # Third question to choose how to download
        download_select = questionary.select(
            "Select item", choices=["Single Download", "Batch Download", "Exit"]
        ).ask()
        if download_select == "Single Download":
            # Fourth question to choose filter.
            sub_select = questionary.select(
                "Select item", choices=["Completed", "Still Airing", "Exit"]
            ).ask()
            if sub_select == "Completed":
                choice = 1
                Animesources().Tokyoinsider().tokyoinsider_search(search_term, choice)
            elif sub_select == "Still Airing":
                choice = 2
                Animesources().Tokyoinsider().tokyoinsider_search(search_term, choice)
            else:
                pass
        elif download_select == "Batch Download":
            # Fourth question to choose filter.
            sub_select = questionary.select(
                "Select item", choices=["Completed", "Still Airing", "Exit"]
            ).ask()
            if sub_select == "Completed":
                choice = 1
                Animesources().Tokyoinsider().tokyoinsider_batch(search_term, choice)
            elif sub_select == "Still Airing":
                choice = 2
                Animesources().Tokyoinsider().tokyoinsider_batch(search_term, choice)
            else:
                pass
        else:
            pass
    elif select == "Nyaa":
        Animesources().Nyaa().nyaa_search(search_term)
    elif select == "Kayoanime":
        Animesources().Kayoanime().kayoanime_search(search_term)
    elif select == "Nibl":
        download_select = questionary.select(
            "Select item", choices=["Single Download", "Batch Download", "Exit"]
        ).ask()
        if download_select == "Single Download":
            Animesources().Nibl().nibl_search(search_term)
        elif download_select == "Batch Download":
            Animesources().Nibl().nibl_batch(search_term)
        else:
            pass
    elif select == "Animk":
        download_select = questionary.select(
            "Select item", choices=["Single Download", "Batch Download", "Exit"]
        ).ask()
        if download_select == "Single Download":
            Animesources().Animk().animk_search(search_term)
        elif download_select == "Batch Download":
            Animesources().Animk().animk_batch(search_term)
        else:
            pass
    elif select == "Reset":
        Core().questions()
    else:
        pass


############################################################################
def books():
    # Second question to choose the Website.
    search_term = input("Enter the title or the author of the Book: ")
    select = questionary.select(
        "Select item",
        choices=[
            "Libgen",
            "Annas Archive",
            "1337x",
            "Rutracker",
            "Golden Audio Books",
            "Reset",
            "Exit",
        ],
    ).ask()
    if select == "Libgen":
        # Third question to choose filter.
        sub_select = questionary.select(
            "Select item", choices=["Search by Title", "Search by Author", "Exit"]
        ).ask()
        if sub_select == "Search by Title":
            choice = 1
            Booksources().Libgen().libgen_search(search_term, choice)
        elif sub_select == "Search by Author":
            choice = 2
            Booksources().Libgen().libgen_search(search_term, choice)
        else:
            pass
    elif select == "Annas Archive":
        # Third question to choose filter.
        sub_select = questionary.select(
            "Select item", choices=["Search by Title", "Search by Author", "Exit"]
        ).ask()
        if sub_select == "Search by Title":
            choice = 1
            Booksources().Annas_Archive().annas_archive_search(search_term, choice)
        elif sub_select == "Search by Author":
            choice = 2
            Booksources().Annas_Archive().annas_archive_search(search_term, choice)
        else:
            pass
    elif select == "1337x":
        Booksources().Torrent().torrent_search(search_term)
    elif select == "Rutracker":
        # Third question to choose filter.
        sub_select = questionary.select(
            "Select item",
            choices=["Search General Courses", "Search Computer Courses", "Exit"],
        ).ask()
        if sub_select == "Search General Courses":
            choice = 1
            Booksources().Rutracker().rutracker(choice)
        elif sub_select == "Search Computer Courses":
            choice = 2
            Booksources().Rutracker().rutracker(choice)
        else:
            pass
    elif select == "Golden Audio Books":
        Booksources().Goldenaudiobooks().goldenaudiobooks(search_term)
    elif select == "Reset":
        Core().questions()
    else:
        pass


############################################################################
def manga():
    # Second question to choose the Website.
    search_term = input("Enter the title of the Manga: ")
    select = questionary.select(
        "Select item",
        choices=[
            "Bato",
            "Weebcentral",
            "Nyaa",
            "Rawotaku",
            "Readallcomics",
            "Reset",
            "Exit",
        ],
    ).ask()
    if select == "Bato":
        Mangasources().Bato().bato_search(search_term)
    elif select == "Weebcentral":
        # Third question to choose how to download
        download_select = questionary.select(
            "Select item", choices=["Single Download", "Batch Download", "Exit"]
        ).ask()
        if download_select == "Single Download":
            Mangasources().Weebcentral().weebcentral(search_term)
        elif download_select == "Batch Download":
            Mangasources().Weebcentral().weebcentral_batch(search_term)
        else:
            pass
    elif select == "Nyaa":
        Mangasources().Nyaa().nyaa_search(search_term)
    elif select == "Rawotaku":
        Mangasources().Rawotaku().rawotaku(search_term)
    elif select == "Readallcomics":
        Mangasources().Readallcomics().readallcomics(search_term)
    elif select == "Reset":
        Core().questions()
    else:
        pass


############################################################################
def music():
    # Second question to choose the Website.
    search_term = input("Enter the title of the Music or Podcast: ")
    select = questionary.select(
        "Select item",
        choices=[
            "Tancpol",
            "Podcast Index",
            "PlayerFM",
            "Youtube Music",
            "1337x",
            "Reset",
            "Exit",
        ],
    ).ask()
    if select == "Podcast Index":
        Musicsources().Podcastindex().podcastindex(search_term)
    elif select == "Tancpol":
        Musicsources().Tancpol().tancpol(search_term)
    elif select == "PlayerFM":
        Musicsources().Player_fm().player(search_term)
    elif select == "Youtube Music":
        Musicsources().Youtubemusic().youtubemusic(search_term)
    elif select == "1337x":
        Musicsources().Torrent().torrent_search(search_term)
    elif select == "Reset":
        Core().questions()
    else:
        pass


############################################################################
def tv():
    # Second question to choose the Website.
    search_term = input("Enter the title of the TV-Series/Movie: ")
    select = questionary.select(
        "Select item",
        choices=[
            "Vadapav",
            "1337x",
            "Documentaries",
            "Vegamovies",
            "Asian Dramas",
            "Reset",
            "Exit",
        ],
    ).ask()
    if select == "Vadapav":
        # Third question to choose how to download
        download_select = questionary.select(
            "Select item", choices=["Single Download", "Batch Download", "Exit"]
        ).ask()
        if download_select == "Single Download":
            Tvsources().Vadapav().vadapav_search()
        elif download_select == "Batch Download":
            Tvsources().Vadapav().vadapav_batch()
        else:
            pass
    elif select == "1337x":
        # Third question to choose the filter
        sub_select = questionary.select(
            "Select item",
            choices=[
                "Search Movies",
                "Search TV-Series",
                "Search Documentaries",
                "Exit",
            ],
        ).ask()
        if sub_select == "Search Movies":
            choice = 1
            Tvsources().Torrent().torrent_search(search_term, choice)
        elif sub_select == "Search TV-Series":
            choice = 2
            Tvsources().Torrent().torrent_search(search_term, choice)
        elif sub_select == "Search Documentaries":
            choice = 3
            Tvsources().Torrent().torrent_search(search_term, choice)
        else:
            pass
    elif select == "Documentaries":
        Tvsources().Documentary().documentary()
    elif select == "Vegamovies":
        Tvsources().Vegamovies().vegamovies(search_term)
    elif select == "Asian Dramas":
        Tvsources().Asian_Dramas().asian_dramas()
    elif select == "Reset":
        Core().questions()
    else:
        pass


############################################################################
def stream():
    # Second question to choose the Website.
    search_term = input("Enter the title of the media you want to Stream: ")
    select = questionary.select(
        "Select item",
        choices=[
            "Anizone",
            "Miruro",
            "Hydrahd",
            "Cxtvlive",
            "IPTV",
            "Reset",
            "Exit",
        ],
    ).ask()
    if select == "Anizone":
        Streamsources().Anizone().anizone(search_term)
    elif select == "Miruro":
        Streamsources().Miruro().miruro(search_term)
    elif select == "Hydrahd":
        Streamsources().Hydrahd().hydrahd(search_term)
    elif select == "Cxtvlive":
        Streamsources().Cxtvlive().cxtvlive()
    elif select == "IPTV":
        sub_select = questionary.select(
            "Select item", choices=["Global IPTV", "South Asian IPTV", "Exit"]
        ).ask()
        if sub_select == "Global IPTV":
            choice = 1
            Streamsources().Iptv().iptv(choice)
        elif sub_select == "South Asian IPTV":
            choice = 2
            Streamsources().Iptv().iptv(choice)
        else:
            pass
    elif select == "Reset":
        Core().questions()
    else:
        pass


############################################################################
class Core:
    def questions(self):
        """Function to start the core app."""
        # First question to choose the medium.
        type = questionary.select(
            "Select item",
            choices=["Anime", "Book", "Manga", "Music", "TV/Movie", "Stream", "Exit"],
        ).ask()
        if type == "Anime":
            anime()
        elif type == "Book":
            books()
        elif type == "Manga":
            manga()
        elif type == "Music":
            music()
        elif type == "TV/Movie":
            tv()
        elif type == "Stream":
            stream()
        elif type == "Exit":
            pass

    # Creating and changing to Download folder.
    windows = "/mnt/e/Downloads/"
    android = "/data/data/com.termux/files/home/storage/downloads"
    if not os.path.isdir(windows):
        os.mkdir(windows)
        os.chdir(windows)
    else:
        os.chdir(windows)
    dir = "Downitall"
    if not os.path.isdir(dir):
        os.mkdir(dir)
        os.chdir(dir)
    else:
        os.chdir(dir)

    # General UI and introduction with guide.
    logo = """
    ########   #######   #######  ##    ##  ######  ##       ####
    ##     ## ##     ## ##     ## ##   ##  ##    ## ##        ## 
    ##     ## ##     ## ##     ## ##  ##   ##       ##        ## 
    ########  ##     ## ##     ## #####    ##       ##        ## 
    ##     ## ##     ## ##     ## ##  ##   ##       ##        ## 
    ##     ## ##     ## ##     ## ##   ##  ##    ## ##        ## 
    ########   #######   #######  ##    ##  ######  ######## ####
    \n\n
    
    Welcome, here is a short guide about websites
    
    (*) = Website uses Torrent, (-) = Website does not have download capablities
    
    Anime: TokyoInsider, Nyaa(*), Kayoanime, Nibl, Animk
    Books: Libgen, Annas Archive(-), 1337x(*), Rutracker(*), Golden Audio Books
    Manga: Bato, Weebcentral, Rawotaku, Nyaa(*), Readallcomics
    Music: Tancpol, Podcast Index, PlayerFM, 1337x(*), Youtube Music
    TV-Series/Movies: Vadapav, 1337x(*), Documentaries, Vegamovies, Asian Dramas
    Stream: Anizone, Miruro, Hydrahd, Cxtvlive, IPTV
    """
    subprocess.call(["clear"])
    print(logo.center(20))


choice = 0



"""Music Sources"""

# Importing necessary modules
import os
import re
import subprocess
import time

import requests
from bs4 import BeautifulSoup
from downitall.resources.globalfunctions import Globalfunctions, Music
from selenium.common.exceptions import (SessionNotCreatedException,
                                        TimeoutException, WebDriverException)
from ytmusicapi import YTMusic

driver = Globalfunctions().webdriver()[0]
bookcli = Globalfunctions().webdriver()[1]


class Musicsources:

    ############################################################################
    ############################################################################
    class Podcastindex:
        def podcastindex(self, search_term):
            Globalfunctions().directory(name="Music")
            # Declaring function level variables.
            label = "next_only"
            # Making search term better for url through regex.
            term = re.sub(r"\W", "+", search_term)
            # Url to access webpage.
            web_url = f"https://podcastindex.org/search?q={term}&type=all"
            # Url to be used for page navigation.
            # Url to access the base website
            base_url = "https://podcastindex.org"
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                time.sleep(3)
                # Getting html page with BeautifulSoup module
                index_list = []
                name_list = []
                link_list = []
                index_list.append(0)
                name_list.append("Next Page")
                link_list.append("")
                while True:
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all("div", class_="result-title")
                    # Finding the links and making link list.
                    for i, tag in enumerate(html_tag, start=1):
                        index_list.append(i)
                        for links in tag.find_all("a"):
                            name_list.append(links.text)
                            link_list.append(base_url + links["href"])
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    # Calling the user choice function.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if name == "Next Page":
                        driver.execute_script(
                            "window.scrollTo(0, document.body.scrollHeight);"
                        )
                        time.sleep(5)
                    else:
                        break
                driver.get(url)
                time.sleep(3)
                index_list = []
                name_list = []
                link_list = []
                index_list.append(0)
                name_list.append("Next Page")
                link_list.append("")
                while True:
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all("a", class_="episode-link")
                    name_tag = soup.find_all("div", class_="episode-title")
                    # Finding the links and making link list.
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        link_list.append(links["href"])
                    for names in name_tag:
                        name_list.append(names.text)
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    # Calling the user choice function.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if name == "Next Page":
                        driver.execute_script(
                            "window.scrollTo(0, document.body.scrollHeight);"
                        )
                        time.sleep(2)
                    else:
                        name = data[1] + ".mp3"
                        break
                Music().download(name, url)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Tancpol:
        def tancpol(self, search_term):
            Globalfunctions().directory(name="Music")
            # Declaring function level variables.
            label = "no_page"
            # Url to access webpage.
            web_url = f"https://st-tancpol.ru/music/{search_term}"
            # Url to access the base website
            base_url = "https://st-tancpol.ru"
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                html_tag = soup.find_all("p", class_="item-subtitle nowrap pajax-link")
                index_list = []
                name_list = []
                link_list = []
                # Finding the links and making link list.
                for i, links in enumerate(html_tag, start=1):
                    index_list.append(i)
                    link_list.append(base_url + links["href"])
                    name_list.append(links.text)
                print("\n")
                for index_name, name in zip(index_list, name_list):
                    # Printing name with index for user to choose.
                    print(f"{index_name}. {name}")
                # Calling the user choice function.
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                url = data[0]
                name = data[1] + ".mp3"
                driver.get(web_url)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                url = (
                    "https:"
                    + soup.find(
                        "div",
                        class_="item-track fx-row fx-middle song playitem track-item",
                    )["data-file"]
                )
                print(url)
                Music().download(name, url)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Player_fm:
        def player(self, search_term):
            Globalfunctions().directory(name="Music")
            # Declaring function level variables.
            label = "next_only"
            # Making search term better for url through regex.
            term = re.sub(r"\W", "+", search_term)
            # Url to access webpage.
            web_url = f"https://player.fm/search/{term}"
            # Url to access the base website
            base_url = "https://player.fm"
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                while True:
                    index_list = []
                    name_list = []
                    link_list = []
                    # Finding the links and making link list.
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append("")
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all(["h3", "h2"], class_="title")
                    for i, links in enumerate(html_tag, start=1):
                        link = links.find("a")
                        index_list.append(i)
                        name_list.append(link.text.strip())
                        link_list.append(base_url + link["href"].strip())
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    # Calling the user choice function.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if name == "Next Page":
                        driver.execute_script(
                            "window.scrollTo(0, document.body.scrollHeight);"
                        )
                        time.sleep(5)
                    else:
                        break
                driver.get(url)
                index_list = []
                name_list = []
                link_list = []
                index_list.append(0)
                name_list.append("Next Page")
                link_list.append("")
                while True:
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    html_tag = soup.find_all("main")[3]
                    html_tag_name = html_tag.find_all("h3", class_="title")
                    html_tag_link = html_tag.find_all(
                        "a", class_="normal action playable round-button info-right"
                    )
                    for i, tag in enumerate(html_tag_name, start=1):
                        for names in tag.find_all("a"):
                            index_list.append(i)
                            name_list.append(names.text.strip())
                    for links in html_tag_link:
                        link_list.append(links["href"])
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if name == "Next Page":
                        scroll_amount = 1000
                        driver.execute_script(f"window.scrollBy(0, {scroll_amount})")
                        time.sleep(2)
                    else:
                        name = re.sub(r"\W", "_", data[1]) + ".mp3"
                        break
                Music().download(name, url)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Youtubemusic:
        def youtubemusic(self, search_term):
            ytmusic = YTMusic()
            Globalfunctions().directory(name="Music")
            try:
                base_url = "https://music.youtube.com/watch?v="
                label = "no_page"
                search = ytmusic.search(search_term, "songs", limit=50)
                index_list = []
                name_list = []
                link_list = []
                for i, element in enumerate(search, start=1):
                    index_list.append(i)
                    name_list.append(
                        element["title"] + " - " + element["album"]["name"]
                    )
                    link_list.append(base_url + element["videoId"])
                print("\n")
                for index_name, name in zip(index_list, name_list):
                    # Printing name with index for user to choose.
                    print(f"{index_name}. {name}")
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                url = data[0]
                name = data[1]
                new_name = re.sub("[^a-z,0-9]", "_", name, flags=re.IGNORECASE)
                subprocess.call(
                    ["yt-dlp", "-x", "--audio-format", "mp3", "-o", new_name, url]
                )
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Torrent:
        def torrent_search(self, search_term):
            Globalfunctions().directory(name="Music")
            label = "paged"
            index = 1
            term = re.sub(r"\W", "+", search_term)
            # Url to access the searching.
            web_url = (
                f"https://1337x.to/sort-category-search/{term}/Music/seeders/desc/1/"
            )
            page_url = (
                f"https://1337x.to/sort-category-search/{term}/Music/seeders/desc/"
            )
            # Url to access the base website
            base_url = "https://1337x.to"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    if Globalfunctions().exists(element=".pagination"):
                        html_tag = soup.find("div", class_="pagination")
                        if html_tag.find("li", class_="last"):
                            tag = html_tag.find("li", class_="last")
                            pages = int(
                                tag.find("a")["href"].split("desc/")[1].strip("/")
                            )
                        else:
                            pages = 1
                    else:
                        pages = 1
                    # Finding all the mentioned elements from webpage.
                    table = soup.find(
                        "table",
                        class_="table-list table table-responsive table-striped",
                    )
                    df = Globalfunctions().visual_tables(table)
                    df_table = df.iloc[:, [0, 1]]
                    df_table.index += 1
                    index_list = df_table.index.tolist()
                    name_list = df[df.columns[0]].values.tolist()
                    link_list = []
                    td_tag = table.find_all("td", class_="coll-1 name")
                    for a_tag in td_tag:
                        links = a_tag.find("a", text=True)
                        link_list.append(base_url + links["href"])
                    index_list.insert(0, 0)
                    name_list.insert(0, "Next Page")
                    link_list.insert(0, page_url)
                    index_list.append(len(df_table.index.tolist()) + 1)
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    index = Globalfunctions().page_navigation(name, pages, index)
                    web_url = url + str(index) + "/"
                    if name != "Next Page" and name != "Previous Page":
                        break
                # Sending get request to the website.
                driver.get(url)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements in the webpage.
                html_tag = soup.find("a", id="id1")
                url = html_tag["href"]
                dir = os.getcwd()
                # Calling the aria2c module to download.
                args = [
                    "aria2c",
                    "--file-allocation=none",
                    "--seed-time=0",
                    "-d",
                    dir,
                    url,
                ]
                subprocess.call(args)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")


############################################################################



"""TV Sources"""

# Importing necessary modules
import os
import re
import subprocess
import time

import requests
from bs4 import BeautifulSoup
from downitall.resources.globalfunctions import Globalfunctions
from selenium.common.exceptions import (
    NoSuchElementException,
    SessionNotCreatedException,
    TimeoutException,
    WebDriverException,
)
from selenium.webdriver.common.by import By

driver = Globalfunctions().webdriver()[0]
bookcli = Globalfunctions().webdriver()[1]


class Tvsources:
    ############################################################################
    ############################################################################
    class Vadapav:
        def vadapav_search(self):
            Globalfunctions().directory(name="TV Movies")
            label = "no_page"
            # Url to access the searching.
            url = "https://vadapav.mov/"
            # Url to access the base website.
            base_url = "https://vadapav.mov"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all(
                        "a", {"class": ["directory-entry wrap", "file-entry wrap"]}
                    )
                    print(html_tag)
                    index_list = []
                    name_list = []
                    link_list = []
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        name_list.append(links.text.strip())
                        link_list.append(base_url + links["href"])
                    print("\n")
                    for i, name in zip(index_list, name_list):
                        print(f"{i}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if "mkv" in url:
                        break
                # Downloading the file with wget as it is fast and has its own progress bar.
                args = ["wget", "-O", name, url]
                subprocess.call(args)
                print("\nDownload Complete.")
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

        ############################################################################
        def vadapav_batch(self):
            Globalfunctions().directory(name="TV Movies")
            label = "no_page"
            save = 0
            # Url to access the searching.
            url = "https://vadapav.mov/s/"
            # Url to access the base website.
            base_url = "https://vadapav.mov"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all("a", {"class": "directory-entry wrap"})
                    index_list = []
                    name_list = []
                    link_list = []
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        name_list.append(links.text.strip())
                        link_list.append(base_url + links["href"])
                    if save == 1:
                        name_list.pop(0)
                        link_list.pop(0)
                    for i, name in zip(index_list, name_list):
                        print(f"{i}. {name}")
                    if not link_list:
                        break
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if save == 0:
                        tv_name = name
                    save += 1
                # Sending request to the webpage.
                driver.get(url)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                html_tag = soup.find_all("a", {"class": "file-entry wrap"})
                link_list = []
                name_list = []
                for links in html_tag:
                    link_list.append(base_url + links["href"])
                    name_list.append(links.text.strip())
                total_links = len(link_list)
                if not os.path.isdir(tv_name):
                    os.mkdir(tv_name)
                    os.chdir(tv_name)
                else:
                    os.chdir(tv_name)
                if os.path.isfile(f".{tv_name}.txt"):
                    file = open(f".{tv_name}.txt", "r+")
                    data = file.read()
                    start = len(data)
                else:
                    open(f".{tv_name}.txt", "x")
                    file = open(f".{tv_name}.txt", "r+")
                    start = 0
                for ep_name in name_list[start:]:
                    for url in link_list[start:]:
                        args = ["wget", "-O", ep_name, url]
                        subprocess.call(args)
                        link_list.pop(0)
                        new_total_links = len(link_list)
                        file.write(str(total_links - new_total_links))
                        data = file.read()
                        start = len(data)
                        break
                print("\nDownload Complete.")
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Torrent:
        def torrent_search(self, search_term, choice):
            Globalfunctions().directory(name="TV Movies")
            label = "paged"
            index = 1
            term = re.sub(r"\W", "+", search_term)
            if choice == 1:
                search_type = "Movies"
            elif choice == 2:
                search_type = "TV"
            else:
                search_type = "Documentaries"
            # Url to access the searching.
            web_url = f"https://1337x.to/sort-category-search/{term}/{search_type}/seeders/desc/1/"
            page_url = f"https://1337x.to/sort-category-search/{term}/{search_type}/seeders/desc/"
            # Url to access the base website
            base_url = "https://1337x.to"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    if Globalfunctions().exists(element=".pagination"):
                        html_tag = soup.find("div", class_="pagination")
                        if html_tag.find("li", class_="last"):
                            tag = html_tag.find("li", class_="last")
                            pages = int(
                                tag.find("a")["href"].split("desc/")[1].strip("/")
                            )
                        else:
                            pages = 1
                    else:
                        pages = 1
                    # Finding all the mentioned elements from webpage.
                    table = soup.find(
                        "table",
                        class_="table-list table table-responsive table-striped",
                    )
                    df = Globalfunctions().visual_tables(table)
                    df_table = df.iloc[:, [0, 1]]
                    df_table.index += 1
                    index_list = df_table.index.tolist()
                    name_list = df[df.columns[0]].values.tolist()
                    link_list = []
                    td_tag = table.find_all("td", class_="coll-1 name")
                    for a_tag in td_tag:
                        links = a_tag.find("a", text=True)
                        link_list.append(base_url + links["href"])
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    index_list.append(len(df_table.index.tolist()) + 1)
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    print("\n0. Next Page")
                    print(df_table)
                    print(f"{len(df_table.index.tolist())+1}. Previous Page")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    index = Globalfunctions().page_navigation(name, pages, index)
                    web_url = url + str(index) + "/"
                    if name != "Next Page" and name != "Previous Page":
                        break
                # Sending get request to the website.
                driver.get(url)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements in the webpage.
                html_tag = soup.find("a", id="id1")
                url = html_tag["href"]
                dir = os.getcwd()
                # Calling the aria2c module to download.
                args = [
                    "aria2c",
                    "--file-allocation=none",
                    "--seed-time=0",
                    "-d",
                    dir,
                    url,
                ]
                subprocess.call(args)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Documentary:
        def documentary(self):
            Globalfunctions().directory(name="TV Movies")
            label = "no_page"
            index = 1
            # Url to access the searching.
            web_url = "https://documentaryheaven.com/watch-online/"
            # Url to access the base website
            base_url = "https://documentaryheaven.com"

            try:
                # Sending request to the webpage.
                driver.get(web_url)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                html_tag = soup.find_all("a", class_="browse-all")
                index_list = []
                name_list = []
                link_list = []
                for i, links in enumerate(html_tag, start=1):
                    index_list.append(i)
                    name_list.append(links.text.replace("Browse ", ""))
                    link_list.append(base_url + links["href"])
                print("\n")
                for i, name in zip(index_list, name_list):
                    print(f"{i}. {name}")
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                web_url = data[0]
                page_url = f"{web_url}/page/"
                while True:
                    label = "paged"
                    driver.get(web_url)
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    if Globalfunctions().exists(element=".numeric-nav"):
                        html_tag = soup.find("div", class_="numeric-nav")
                        tag = html_tag.find_all("li", class_=None)[-1]
                        pages = int(tag.find("a").text.strip())
                    else:
                        pages = 1
                    html_tag = soup.find_all("h2")
                    index_list = []
                    name_list = []
                    link_list = []
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        name_list.append(links.text)
                        tag = links.find_all("a")
                        for link in tag:
                            link_list.append(link["href"])
                    index_list.append(i + 1)
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    print("\n")
                    for i, name in zip(index_list, name_list):
                        print(f"{i}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    index = Globalfunctions().page_navigation(name, pages, index)
                    web_url = url + str(index) + "/"
                    if name != "Next Page" and name != "Previous Page":
                        break
                # Sending get request to the website.
                driver.get(url)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements in the webpage.
                html_tag = soup.find("meta", attrs={"itemprop": "embedUrl"})
                if "https://www.dailymotion" in html_tag["content"].split(".com")[0]:
                    url = html_tag["content"].replace("/embed", "")
                else:
                    url = html_tag["content"]
                args = ["yt-dlp", url]
                subprocess.call(args)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Vegamovies:
        def vegamovies(self, search_term):
            Globalfunctions().directory(name="TV Movies")
            label = "no_page"
            # Url to access the searching.
            web_url = f"https://vegamovies.com.lv/search.php?query={search_term}"
            base_url = "https://vegamovies.com.lv"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all("p", class_="font-medium text-gray-800")
                    index_list = []
                    name_list = []
                    link_list = []
                    for i, tag in enumerate(html_tag, start=1):
                        index_list.append(i)
                        for links in tag.find_all("a"):
                            name_list.append(links.text.strip())
                            link_list.append(base_url + links["href"])
                    for i, name in zip(index_list, name_list):
                        print(f"{i}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if name != "Next Page" and name != "Previous Page":
                        break
                driver.get(url)
                time.sleep(2)
                soup = BeautifulSoup(driver.page_source, "html.parser")
                try:
                    if driver.find_element(By.CSS_SELECTOR, "a.bg-blue-600"):
                        url = (
                            soup.find(
                                "a",
                                class_="rounded-m bg-blue-600 text-white text-[13px] font-bold px-5 py-2 rounded hover:bg-blue-500 hover:bg-blue-700",
                            )["href"]
                            + "?download=main"
                        )
                except NoSuchElementException:
                    url = (
                        soup.find(
                            "a",
                            class_="rounded-m bg-pink-600 text-white text-[13px] font-bold px-5 py-2 rounded hover:bg-pink-500 hover:bg-pink-700",
                        )["href"]
                        + "?download=main"
                    )

                # Downloading the file with wget as it is fast and has its own progress bar.
                args = ["wget", "-O", name, url]
                subprocess.call(args)
                print("\nDownload Complete.")
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except requests.exceptions.RequestException as e:
                print(e)
            except WebDriverException as e:
                print(e)
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Asian_Dramas:
        def asian_dramas(self):
            Globalfunctions().directory(name="TV Movies")
            label = "no_page"
            # Url to access the searching.
            url = "https://dramasuki.pages.dev/#DramaSuki/"
            try:
                driver.get(url)
                while True:
                    # Sending request to the webpage.
                    driver.get(url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all("a", class_="folder_link")
                    span = soup.find("span", class_="file mkv")
                    index_list = []
                    name_list = []
                    link_list = []
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        name_list.append(links.get_text().strip())
                        link_list.append(links.get_text().strip())
                    print("\n")
                    for i, name in zip(index_list, name_list):
                        print(f"{i}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    name = data[1]
                    xpath_expr = f"//a[contains(normalize-space(text()), '{name}')]"
                    element = driver.find_element(By.XPATH, xpath_expr)
                    element.click()
                    time.sleep(2)
                    if span:
                        # Downloading the file with wget as it is fast and has its own progress bar.
                        soup = BeautifulSoup(driver.page_source, "html.parser")
                        # Finding all the mentioned elements from webpage.
                        html_tag = soup.find_all("a", string=lambda t: t and "mkv" in t)
                        index_list = []
                        name_list = []
                        link_list = []
                        for i, links in enumerate(html_tag, start=1):
                            index_list.append(i)
                            name_list.append(links.get_text().strip())
                            link_list.append(links["href"])
                        print("\n")
                        for i, name in zip(index_list, name_list):
                            print(f"{i}. {name}")
                        data = Globalfunctions().user_choice(
                            name_list, link_list, index_list, label
                        )
                        url = data[0]
                        name = data[1]
                        if "mkv" in name:
                            break
                args = ["wget", "-O", name, url]
                subprocess.call(args)
                print("\nDownload Complete.")
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")



"""Global_Functions"""

import os
import re
import shutil
import subprocess
import sys
import time

import pandas as pd
import questionary
import requests
import tqdm
from bs4 import BeautifulSoup
from cbz_generator import create_cbz_archive as cbz_generator
from selenium import webdriver
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By
from term_image.image import from_url
from tqdm.auto import tqdm


class Globalfunctions:
    """Global_Functions"""

    def webdriver(self):
        options = webdriver.ChromeOptions()
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-dev-shm-usage")
        options.add_argument("--headless=new")
        options.add_argument("--disable-renderer-backgrounding")
        options.add_argument("--disable-background-timer-throttling")
        options.add_argument("--disable-backgrounding-occluded-windows")
        options.add_argument("--disable-client-side-phishing-detection")
        options.add_argument("--disable-crash-reporter")
        options.add_argument("--disable-oopr-debug-crash-dump")
        options.add_argument("--no-crash-upload")
        options.add_argument("--disable-gpu")
        options.add_argument("--disable-extensions")
        options.add_argument("--disable-low-res-tiling")
        options.add_argument("--log-level=3")
        options.add_argument("--silent")
        options.page_load_strategy = "eager"
        driver = webdriver.Chrome(options=options)
        bookcli = os.getcwd()
        return driver, bookcli

    ############################################################################
    def directory(self, name):
        if not os.path.isdir(name):
            os.mkdir(name)
            os.chdir(name)
        else:
            os.chdir(name)

    ############################################################################
    # Defining the function for moving through pages, it takes name, pages(total pages) and, index(starting of pages).
    def page_navigation(self, name, pages, index):
        # If statement to confirm what user choose, next or previous page.
        if name == "Next Page":
            # If statement to check whether it isn't the last page, if not add one else reset the count.
            if index < pages:
                index += 1
            else:
                index = 1
        else:
            # If statement to check whether it isn't the first page, if not add one else don't change the count.
            if index != 1:
                index -= 1
            else:
                index = 1
        return index

    ############################################################################
    # Defining the function to check if an element exists in the current page.
    def exists(self, element):
        driver = Globalfunctions().webdriver()[0]
        # Using try except to use the error catching as false argument.
        try:
            driver.find_element(By.CSS_SELECTOR, element)
        except NoSuchElementException:
            return False
        return True

    ############################################################################
    # Defining the function for tables through pandas, table is the element.
    def visual_tables(self, table):
        # Finding the table body from table element.
        tbody = table.find("tbody")
        # Creating data list and row list. Looping to find all table rows and cells while appending cell text to row list and appending row list to data list.
        data = []
        for row in tbody.find_all("tr"):
            row_data = []
            for cell in row.find_all("td"):
                cell_info = cell.text.strip()
                clean_cell = re.sub(r"\s+", " ", cell_info)
                row_data.append(clean_cell[:90])
            data.append(row_data)
        # Creating a pandas dataframe
        df = pd.DataFrame(data)
        pd.set_option("display.max_rows", None)
        return df

    ############################################################################
    # Defining the function for player to choose the index.
    def user_choice(self, name_list, link_list, index_list, label):
        # Appendind data for next and previous page.
        # Creating two dictionary that take key as index and value as items from "link_list" and "name_list" respectively.
        url_dict = {index_list[i]: link_list[i] for i in range(len(link_list))}
        name_dict = {index_list[i]: name_list[i] for i in range(len(name_list))}
        try:
            # If statement in case no manga/chapter was found.
            if label == "no_page" and not name_list:
                print("\nCould not find any thing :(")
                sys.exit()
            elif label == "paged" and len(name_list) == 2:
                print("\nCould not find any thing :(")
                sys.exit()
            elif label == "next_only" and len(name_list) == 1:
                print("\nCould not find any thing :(")
                sys.exit()
            else:
                # Matching the user selection with "urls_dict" dictionary to get its value.
                selection = int(input("\n\nSelect the index number: "))
                if selection in url_dict:
                    # Getting the name and link by matching the index number from dictionaries.
                    link = f"{url_dict[selection]}"
                    name = f"{name_dict[selection]}"
                    print("\nFetching, please wait...")
                    subprocess.call(["clear"])
                    return [link, name, selection]
                else:
                    raise ValueError
        except ValueError:
            print("\nInvalid integer. The number must be in the range.")

    ############################################################################
    def retry(self):
        from downitall.core import Core

        # Using match case argument to see which class called the function.
        answer = questionary.select(
            "Do you want to download another file? ", choices=["Yes", "No"]
        ).ask()
        if answer == "Yes":
            Core().questions()
        else:
            sys.exit()


############################################################################
############################################################################


class Anime:
    """Anime functions"""

    # Defining the function for player to choose the index.
    def xdcc_choice(self, link_list, index_list, label):
        # Appendind data for next and previous page.
        # Creating two dictionary that take key as index and value as items from "link_list" and "name_list" respectively.
        url_dict = {index_list[i]: link_list[i] for i in range(len(link_list))}
        try:
            # If statement in case no manga/chapter was found.
            if label == "no_page" and not link_list:
                print("\nCould not find any thing :(")
                sys.exit()
            elif label == "paged" and len(link_list) == 2:
                print("\nCould not find any thing :(")
                sys.exit()
            elif label == "next_only" and len(link_list) == 1:
                print("\nCould not find any thing :(")
                sys.exit()
            else:
                while True:
                    # Matching the user selection with "urls_dict" dictionary to get its value.
                    start = int(input("\n\nSelect the starting index number: "))
                    end = int(input("\n\nSelect the ending index number: "))
                    if start in url_dict and end in url_dict:
                        # Getting the name and link by matching the index number from dictionaries.
                        start_link = f"{url_dict[start]}"
                        end_link = f"{url_dict[end]}"
                        start_index = start
                        end_index = end
                        print("\nFetching, please wait...")
                        subprocess.call(["clear"])
                        return [start_link, end_link, start_index, end_index]
                    else:
                        raise ValueError
        except ValueError:
            print("\nInvalid integer. The number must be in the range.")

    ############################################################################
    def download(self, name, url):
        bookcli = Globalfunctions().webdriver()[1]
        # Making the folder and opening it
        anime = re.sub("[^a-z,0-9]", "_", name, flags=re.IGNORECASE)
        if not os.path.isdir(anime):
            os.mkdir(anime)
            os.chdir(anime)
        elif bookcli.split("/")[-1] == anime:
            pass
        else:
            os.chdir(anime)
        # Downloading the file with wget as it is fast and has its own progress bar.
        args = ["wget", url]
        subprocess.call(args)
        print("\nDownload Complete.")


############################################################################
############################################################################


class Book:

    def download(self, name, url):
        # Reformatting the book name to a standard name and sending a request to server with connection as active.
        book_name = re.sub("[^a-z,0-9.]", "_", name, flags=re.IGNORECASE)
        with requests.get(url, stream=True) as res:
            # Checking header to get the content length, in bytes.
            total_length = int(res.headers.get("Content-Length"))
            # Checking if request was valid.
            if res.status_code == 200:
                # Implement progress bar via tqdm.
                with tqdm.wrapattr(res.raw, "read", total=total_length, desc="") as raw:
                    # Downloading image via shutil.
                    with open(book_name, "wb") as f:
                        shutil.copyfileobj(raw, f)
                print("\nBook sucessfully Downloaded: ", book_name)
            else:
                print("\nBook couldn't be retrieved")


############################################################################
############################################################################


class Manga:

    def confirmation(self, selection, index_list, img_list):
        img_dict = {
            index_list[i]: img_list[j]
            for i, j in enumerate(range(len(img_list)), start=1)
        }
        img = f"{img_dict[selection]}"
        image = from_url(img)
        subprocess.call(["clear"])
        print(f"\n{image}\n\n")
        answer = questionary.select("Is this the manga?", choices=["Yes", "No"]).ask()
        if answer == "Yes":
            subprocess.call(["clear"])
            return True
        else:
            subprocess.call(["clear"])
            return False

    ############################################################################
    def download_directory(self, manga_name, chapter_name):
        manga = re.sub("[^a-z,0-9]", "_", manga_name, flags=re.IGNORECASE)
        chapter = re.sub("[^a-z,0-9]", "_", chapter_name, flags=re.IGNORECASE)
        if not os.path.isdir(manga):
            os.mkdir(manga)
            os.chdir(manga)
            path = os.getcwd()
            if not os.path.isdir(chapter):
                os.mkdir(chapter)
                os.chdir(chapter)
            else:
                os.chdir(chapter)
        elif os.path.isdir(manga):
            os.chdir(manga)
            path = os.getcwd()
            if not os.path.isdir(chapter):
                os.mkdir(chapter)
                os.chdir(chapter)
            else:
                os.chdir(chapter)
        else:
            os.chdir(manga)
            path = os.getcwd()
            os.chdir(chapter)
        return [path, manga, chapter]

    ############################################################################
    def download_compress(self, manga_name, chapter_name, img_links_list):
        # Using regex to get the standard naming protocols for easy folder making.
        try:
            data = Manga().download_directory(manga_name, chapter_name)
            path = data[0]
            manga = data[1]
            chapter = data[2]
            Manga().download(img_links_list, manga, chapter, path)
            # Accessing the manga folder to compress chapter folder into cbz file.
            os.chdir(path)
            # Compressing chapter folder into cbz file using cbz_generator module.
            cbz_generator.create_cbz_archive(chapter, path, f"{manga}_{chapter}")
            print("\nDownload Complete")
        except OSError as e:
            print("\n", e)

    ############################################################################
    def download(self, img_links_list, manga, chapter, path):
        # Enumerating the "img_links_list" to get each link with index.
        for i, img in enumerate(img_links_list):
            # Sending request to the image link.
            with requests.get(img, stream=True) as res:
                # Checking header to get the content length, in bytes.
                total_length = int(res.headers.get("Content-Length"))
                # Checking if request was valid.
                if res.status_code == 200:
                    # Naming the image and storing it.
                    image = f"{chapter} Image {i}.jpg"
                    # Implement progress bar via tqdm.
                    with tqdm.wrapattr(
                        res.raw, "read", total=total_length, desc=""
                    ) as raw:
                        # Downloading image via shutil.
                        with open(image, "wb") as f:
                            shutil.copyfileobj(raw, f)
                    print(f"Image sucessfully Downloaded: {image}\n")
                else:
                    print("Image Couldn't be retrieved")

    ############################################################################
    def batch_download(self, name_list, link_list, name):
        download_dir = os.getcwd()
        total_links = len(link_list)
        if os.path.isfile(f".{name}.txt"):
            file = open(f".{name}.txt", "r+")
            data = file.read()
            start = len(data)
        else:
            open(f".{name}.txt", "x")
            file = open(f".{name}.txt", "r+")
            start = 0
        for chapter_name in name_list[start:]:
            data = Manga().download_directory(name, chapter_name)
            path = data[0]
            manga = data[1]
            chapter = data[2]
            os.chdir(path)
            if not os.path.isdir("Cbz"):
                os.mkdir("Cbz")
            os.chdir("Cbz")
            cbz = os.getcwd()
            os.chdir(download_dir)
            for url in link_list[start:]:
                driver = Globalfunctions().webdriver()[0]
                driver.get(url)
                time.sleep(2)
                os.chdir(path)
                os.chdir(chapter)
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Getting the working download link from html webpage.
                # Finding all the mentioned elements from webpage.
                html_tag = soup.find_all(
                    "img", src=lambda line: "manga" in line or "comic" in line
                )
                # Creating the list to store image url.
                img_links_list = []
                # Finding all the mentioned elements from img_html_chapter.
                for imgs in html_tag:
                    img_links_list.append(imgs["src"].strip())
                Manga().download(img_links_list, manga, chapter, path)
                link_list.pop(0)
                new_total_links = len(link_list)
                # Accessing the manga folder to compress chapter folder into cbz file.
                os.chdir(path)
                # Compressing chapter folder into cbz file using cbz_generator module.
                cbz_generator.create_cbz_archive(chapter, cbz, f"{manga}_{chapter}")
                os.chdir(download_dir)
                file.write(str(total_links - new_total_links))
                data = file.read()
                start = len(data)
                break
        os.remove(f".{name}.txt")
        print("\nDownload Complete.")


############################################################################
############################################################################


class Music:

    def download(self, name, url):
        # Downloading the file with wget as it is fast and has its own progress bar.
        args = ["wget", "-O", name, f"{url}"]
        subprocess.call(args)
        print("\nDownload Complete.")


############################################################################
############################################################################


class Stream:

    def link_check(self, url):
        try:
            res = requests.get(url)
            if res.status_code == 200:
                answer = questionary.select(
                    "Select item", choices=["Best", "Worst"]
                ).ask()
                if answer == "Best":
                    quality = "best"
                else:
                    quality = "worst"
                ## VLC is for android and MPV is for Window
                # subprocess.call(["vlc", url, quality])
                subprocess.call(["mpv", "--really-quiet", "--fs", url])
                return True
            else:
                print("Dead link")
                return False
        except requests.exceptions.RequestException:
            return False


############################################################################
############################################################################



"""Anime Sources"""

# Importing necessary modules
import os
import re
import subprocess
import time

import gdown
import requests
from bs4 import BeautifulSoup
from downitall.resources.globalfunctions import Anime, Globalfunctions
from selenium.common.exceptions import (SessionNotCreatedException,
                                        TimeoutException, WebDriverException)
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
from xdcc_dl.entities import IrcServer, XDCCPack
from xdcc_dl.xdcc import download_packs

driver = Globalfunctions().webdriver()[0]
bookcli = Globalfunctions().webdriver()[1]


class Animesources:
    ########################################################################################################
    ########################################################################################################
    class Kayoanime:
        def kayoanime_search(self, search_term):
            Globalfunctions().directory(name="Anime")
            label = "next_only"
            web_url = f"https://kayoanime.com/?s={search_term}"
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                # Getting html page with BeautifulSoup module
                while True:
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find("div", class_="container-wrapper").find_all(
                        "a", class_="post-thumb", attrs={"aria-label": True}
                    )
                    # Creating lists and appending next page.
                    index_list = []
                    name_list = []
                    link_list = []
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append("")
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        name_list.append(links["aria-label"].strip())
                        link_list.append(links["href"])
                    # Printing the index anf name for user to choose from.
                    print("\n")
                    for i, names in zip(index_list, name_list):
                        print(f"{i}. {names}")
                    # Calling user_choice function to get list called data and use the user selected options.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if name == "Next Page":
                        # Calling exists function to check if an element exists.
                        if (
                            Globalfunctions().exists(
                                element=".container-wrapper.show-more-button.load-more-button.infinite-scroll-archives"
                            )
                            == True
                        ):
                            WebDriverWait(driver, 1).until(
                                EC.element_to_be_clickable(
                                    (
                                        By.CSS_SELECTOR,
                                        ".container-wrapper.show-more-button.load-more-button.infinite-scroll-archives",
                                    )
                                )
                            ).click()
                        # Finding the last element on the page by TAG to scroll to the bottom of the page.
                        elem = driver.find_element(By.TAG_NAME, "html")
                        elem.send_keys(Keys.END)
                        time.sleep(2)
                    else:
                        break
                # Sending get request to the "manga_link" website.
                driver.get(url)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements in the webpage.
                html_tag = soup.find("a", target="_blank")
                # Getting the url by spliting the string by ? and getting the first part.
                url = html_tag["href"].split("?")[0]
                # Downloading google folders using gdown libraries.
                gdown.download_folder(url)
                # Calling retry function to start this entire method again.
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except (
                RuntimeError,
                gdown.exceptions.FileURLRetrievalError,
                gdown.exceptions.FolderContentsMaximumLimitError,
                PermissionError,
            ):
                print("\nGoogle file is either private or unavilable")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    class Tokyoinsider:
        def tokyoinsider_search(self, search_term, choice):
            # Declaring function level variables.
            Globalfunctions().directory(name="Anime")
            label = "paged"
            index = 1
            # Making search term better for url through regex.
            term = re.sub(r"\W", "+", search_term)
            # Checking weather search if for title or author/
            if choice == 1:
                search_type = "&s=completed"
            else:
                search_type = "&s=airing"
            # Url to access the website.
            web_url = f"https://www.tokyoinsider.com/anime/search?k={term}{search_type}"
            # Url to be used for page navigation.
            page_url = web_url + "&start="
            # Url to access the base website
            base_url = "https://www.tokyoinsider.com"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Checking weather the element exists.
                    if Globalfunctions().exists(element=".pager"):
                        html_tag = soup.find("div", class_="pager")
                        # Finding the last td tag and getting its text which is the number of last page and converting it into integer.
                        pages = int(html_tag.find_all("a", class_=None)[-1].getText())
                    else:
                        pages = 1
                    # Finding all the mentioned elements from webpage.
                    table = soup.find(
                        "table",
                        attrs={
                            "width": "100%",
                            "border": "0",
                            "cellspacing": "0",
                            "cellpadding": "3",
                        },
                    )
                    # Calling the table function.
                    df = Globalfunctions().visual_tables(table)
                    # Selecting columns that user can see.
                    df_table = df.iloc[:, [1]]
                    # Making table index start from 1.
                    df_table.index += 1
                    # Converting index column into index list.
                    link_list = []
                    index_list = df_table.index.tolist()
                    name_list = df[df.columns[1]].values.tolist()
                    # Finding the links and making link list.
                    html_tag = soup.find_all(
                        "a", title=True, attrs={"style": "display: block"}
                    )
                    for links in html_tag:
                        link_list.append(base_url + links["href"])
                    # Appending next page on 0 index and previous page on last index i+1.
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    index_list.append(len(df_table.index.tolist()) + 1)
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    # Printing the index and name for user to choose from.
                    print("\n0. Next Page")
                    print(df_table)
                    print(f"{len(df_table.index.tolist())+1}. Previous Page")
                    # Calling the user choice function.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    # Calling the page_navigation function to get new index number for pages.
                    index = Globalfunctions().page_navigation(name, pages, index)
                    if pages == 1:
                        web_url = url + "0"
                    else:
                        # This website uses item numbers instead of pages so here multiplication 20 is necessary.
                        web_url = url + str((index - 1) * 20)
                    if name != "Next Page" and name != "Previous Page":
                        # Sending get request to the website.
                        label = "no_page"
                        driver.get(url)
                        # Parsering the response with "BeauitifulSoup".
                        soup = BeautifulSoup(driver.page_source, "html.parser")
                        # Finding all the mentioned elements in the webpage.
                        html_tag = soup.find_all("a", class_="download-link")
                        # Making lists and appending to them the data.
                        index_list = []
                        name_list = []
                        link_list = []
                        for i, links in enumerate(html_tag, start=1):
                            index_list.append(i)
                            name_list.append(links.text.strip())
                            link_list.append(base_url + links["href"])
                        # Reversing the name and link list to have better organized data.
                        name_list.reverse()
                        link_list.reverse()
                        # Printing the index and name for user to choose from.
                        print("\n")
                        for i, names in zip(index_list, name_list):
                            print(f"{i}. {names}")
                        # Calling the user choice function.
                        data = Globalfunctions().user_choice(
                            name_list, link_list, index_list, label
                        )
                        url = data[0]
                        # Sending request with selenium webdriver.
                        label = "no_page"
                        driver.get(url)
                        # Parsering the response with "BeauitifulSoup".
                        soup = BeautifulSoup(driver.page_source, "html.parser")
                        # Getting the working download link from html webpage.
                        html_tag = soup.find_all("a", href=lambda t: t and "media" in t)
                        index_list = []
                        name_list = []
                        link_list = []
                        for i, links in enumerate(html_tag, start=1):
                            index_list.append(i)
                            name_list.append(links.text.strip())
                            link_list.append(links["href"])
                        name_list.reverse()
                        link_list.reverse()
                        print("\n")
                        for i, names in zip(index_list, name_list):
                            print(f"{i}. {names}")
                        data = Globalfunctions().user_choice(
                            name_list, link_list, index_list, label
                        )
                        url = data[0]
                        Anime().download(name, url)
                        Globalfunctions().retry()
                        break
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

        ############################################################################
        def tokyoinsider_batch(self, search_term, choice):
            # Declaring function level variables.
            Globalfunctions().directory(name="Anime")
            label = "paged"
            index = 1
            # Making search term better for url through regex.
            term = re.sub(r"\W", "+", search_term)
            # Checking the filter used.
            if choice == 1:
                search_type = "&s=completed"
            else:
                search_type = "&s=airing"
            # Url to access the website.
            web_url = f"https://www.tokyoinsider.com/anime/search?k={term}{search_type}"
            # Url to be used for page navigation.
            page_url = web_url + "&start="
            # Url to access the base website
            base_url = "https://www.tokyoinsider.com"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Checking weather the element exists.
                    if Globalfunctions().exists(element=".pager"):
                        html_tag = soup.find("div", class_="pager")
                        # Finding the last td tag and getting its text which is the number of last page and converting it into integer.
                        pages = int(html_tag.find_all("a", class_=None)[-1].getText())
                    else:
                        pages = 1
                    # Finding all the mentioned elements from webpage.
                    table = soup.find(
                        "table",
                        attrs={
                            "width": "100%",
                            "border": "0",
                            "cellspacing": "0",
                            "cellpadding": "3",
                        },
                    )
                    # Calling the table function.
                    df = Globalfunctions().visual_tables(table)
                    # Selecting columns that user can see.
                    df_table = df.iloc[:, [1]]
                    # Making table index start from 1.
                    df_table.index += 1
                    # Converting index column into index list.
                    link_list = []
                    index_list = df_table.index.tolist()
                    name_list = df[df.columns[1]].values.tolist()
                    # Finding the links and making link list.
                    html_tag = soup.find_all(
                        "a", title=True, attrs={"style": "display: block"}
                    )
                    for links in html_tag:
                        link_list.append(base_url + links["href"])
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    index_list.append(len(df_table.index.tolist()) + 1)
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    print("\n0. Next Page")
                    print(df_table)
                    print(f"{len(df_table.index.tolist())+1}. Previous Page")
                    # Calling the user choice function.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    index = Globalfunctions().page_navigation(name, pages, index)
                    if pages == 1:
                        web_url = url + "0"
                    else:
                        web_url = url + str((index - 1) * 20)
                    if name != "Next Page" and name != "Previous Page":
                        label = "no_page"
                        driver.get(url)
                        # Parsering the response with "BeauitifulSoup".
                        soup = BeautifulSoup(driver.page_source, "html.parser")
                        # Finding all the mentioned elements in the webpage.
                        html_tag = soup.find_all("a", class_="download-link")
                        # Making lists and appending to them the data.
                        index_list = []
                        name_list = []
                        link_list = []
                        for i, links in enumerate(html_tag, start=1):
                            index_list.append(i)
                            name_list.append(links.text.strip())
                            link_list.append(base_url + links["href"])
                        # Reversing the name and link list to have better organized data.
                        name_list.reverse()
                        link_list.reverse()
                        # Printing the index and name for user to choose from.
                        print("\n")
                        for i, names in zip(index_list, name_list):
                            print(f"{i}. {names}")
                        # Calling the user choice function.
                        data = Anime().xdcc_choice(link_list, index_list, label)
                        start = int(data[2]) - 1
                        end = int(data[3])
                        anime = re.sub("[^a-z,0-9]", "_", name, flags=re.IGNORECASE)
                        if not os.path.isdir(anime):
                            os.mkdir(anime)
                            os.chdir(anime)
                        else:
                            os.chdir(anime)
                        for url in link_list[start:end]:
                            driver.get(url)
                            soup = BeautifulSoup(driver.page_source, "html.parser")
                            # Getting the working download link from html webpage.
                            html_tag = soup.find_all(
                                "a", href=lambda t: t and "media" in t
                            )[0]
                            url = html_tag["href"]
                            # Downloading the file with wget as it is fast and has its own progress bar.
                            args = ["wget", url]
                            subprocess.call(args)
                            print("\nDownload Complete.")
                        Globalfunctions().retry()
                        break
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Nyaa:
        def nyaa_search(self, search_term):
            Globalfunctions().directory(name="Anime")
            label = "paged"
            index = 1
            term = re.sub(r"\W", "+", search_term)
            # Url to access the searching.
            web_url = f"https://nyaa.si/?q={term}&f=0&c=1_2&s=seeders&o=desc"
            page_url = f"https://nyaa.si/?q={term}&f=0&c=1_2&s=seeders&o=desc&p="
            # Url to access the base website
            base_url = "https://nyaa.si"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    if Globalfunctions().exists(element=".pagination"):
                        html_tag = soup.find("ul", class_="pagination")
                        pages = int(
                            html_tag.find_all("li", class_=None)[-1].getText().strip()
                        )
                    else:
                        pages = 1
                    # Finding all the mentioned elements from webpage.
                    table = soup.find(
                        "table",
                        class_="table table-bordered table-hover table-striped torrent-list",
                    )
                    df = Globalfunctions().visual_tables(table)
                    df_table = df.iloc[:, [1, 3]]
                    df_table.index += 1
                    index_list = df_table.index.tolist()
                    name_list = df[df.columns[1]].values.tolist()
                    link_list = []
                    td_tag = soup.find_all("td", colspan="2")
                    for a_html in td_tag:
                        for links in a_html.find_all(
                            "a", title=True, class_=None, href=True
                        ):
                            link_list.append(base_url + links["href"])
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    index_list.append(len(df_table.index.tolist()) + 1)
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    print("\n0. Next Page")
                    print(df_table)
                    print(f"{len(df_table.index.tolist())+1}. Previous Page")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    index = Globalfunctions().page_navigation(name, pages, index)
                    web_url = url + str(index)
                    if name != "Next Page" and name != "Previous Page":
                        break
                driver.get(url)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the "a" elements in the webpage.
                html_tag = soup.find("a", class_="card-footer-item")
                driver.quit()
                dir = os.getcwd()
                url = html_tag["href"]
                args = [
                    "aria2c",
                    "--file-allocation=none",
                    "--seed-time=0",
                    "-d",
                    dir,
                    url,
                ]
                subprocess.call(args)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Nibl:
        def nibl_search(self, search_term):
            Globalfunctions().directory(name="Anime")
            label = "no_page"
            term = re.sub(r"\W", "+", search_term)
            # Url to access the searching.
            web_url = f"https://nibl.co.uk/search?query={term}"
            # Url to access the base website
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                table = soup.find(
                    "table",
                    class_="border-collapse table-auto w-full whitespace-no-wrap bg-white table-striped relative shadow dark:bg-gray-800",
                )
                df = Globalfunctions().visual_tables(table)
                df_table = df.iloc[:, [0, 3, 2]]
                df_table.index += 1
                index_list = df_table.index.tolist()
                name_list = df[df.columns[3]].values.tolist()
                link_list = []
                html_tag = soup.find_all(
                    "button",
                    class_="copy-data py-1 px-2 bg-gray-300 hover:bg-gray-200 dark:bg-gray-900 dark:hover:bg-gray-800",
                )
                for links in html_tag:
                    link_list.append(
                        links["data-botname"] + " " + links["data-botpack"]
                    )
                print(df_table)
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                bot = data[0].split(" ")[0]
                pack = int(data[0].split(" ")[1])
                manual = XDCCPack(IrcServer("irc.rizon.net"), f"{bot}", pack)
                # Start download
                download_packs([manual])
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

        ############################################################################
        def nibl_batch(self, search_term):
            Globalfunctions().directory(name="Anime")
            label = "no_page"
            term = re.sub(r"\W", "+", search_term)
            # Url to access the searching.
            web_url = f"https://nibl.co.uk/search?query={term}"
            # Url to access the base website
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                table = soup.find(
                    "table",
                    class_="border-collapse table-auto w-full whitespace-no-wrap bg-white table-striped relative shadow dark:bg-gray-800",
                )
                df = Globalfunctions().visual_tables(table)
                df_table = df.iloc[:, [0, 3, 2]]
                df_table.index += 1
                index_list = df_table.index.tolist()
                link_list = []
                html_tag = soup.find_all(
                    "button",
                    class_="copy-data py-1 px-2 bg-gray-300 hover:bg-gray-200 dark:bg-gray-900 dark:hover:bg-gray-800",
                )
                for links in html_tag:
                    link_list.append(
                        links["data-botname"] + " " + links["data-botpack"]
                    )
                print(df_table)
                data = Anime().xdcc_choice(link_list, index_list, label)
                start_bot = data[0].split(" ")[0]
                end_bot = data[1].split(" ")[0]
                start_pack = int(data[0].split(" ")[1])
                end_pack = int(data[1].split(" ")[1])
                while True:
                    if start_bot == end_bot:
                        manual = XDCCPack(
                            IrcServer("irc.rizon.net"), f"{start_bot}", start_pack
                        )
                        from_message = XDCCPack.from_xdcc_message(
                            f"/msg {start_bot} xdcc send #{start_pack + 1}-{end_pack}",
                            os.getcwd(),
                        )
                        combined = [manual] + from_message
                        # Start download
                        download_packs(combined)
                        Globalfunctions().retry()
                    else:
                        print("Make sure to choose from single bot")
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Animk:
        def animk_search(self, search_term):
            Globalfunctions().directory(name="Anime")
            label = "no_page"
            # Url to access the searching.
            web_url = f"https://xdcc.animk.info/?search={search_term}"
            # Url to access the base website
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                time.sleep(4)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                table = soup.find("table", id="listtable")
                df = Globalfunctions().visual_tables(table)
                df_table = df.iloc[:, [0, 3, 2]]
                df_table.index += 1
                index_list = df_table.index.tolist()
                name_list = df[df.columns[3]].values.tolist()
                df["combined_column"] = (
                    df[df.columns[0]] + " " + df[df.columns[1]].astype(str)
                )
                link_list = df["combined_column"].values.tolist()
                print(df_table)
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                bot = data[0].split(" ")[0]
                pack = int(data[0].split(" ")[1])
                manual = XDCCPack(IrcServer("irc.rizon.net"), f"{bot}", pack)
                # Start download
                download_packs([manual])
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

        ############################################################################
        def animk_batch(self, search_term):
            Globalfunctions().directory()
            label = "no_page"
            # Url to access the searching.
            web_url = f"https://xdcc.animk.info/?search={search_term}"
            # Url to access the base website
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                time.sleep(6)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                table = soup.find("table", id="listtable")
                df = Globalfunctions().visual_tables(table)
                df_table = df.iloc[:, [0, 3, 2]]
                df_table.index += 1
                index_list = df_table.index.tolist()
                df["combined_column"] = (
                    df[df.columns[0]] + " " + df[df.columns[1]].astype(str)
                )
                link_list = df["combined_column"].values.tolist()
                print(df_table)
                data = Anime().xdcc_choice(link_list, index_list, label)
                start_bot = data[0].split(" ")[0]
                end_bot = data[1].split(" ")[0]
                start_pack = int(data[0].split(" ")[1])
                end_pack = int(data[1].split(" ")[1])
                while True:
                    if start_bot == end_bot:
                        manual = XDCCPack(
                            IrcServer("irc.rizon.net"), f"{start_bot}", start_pack
                        )
                        from_message = XDCCPack.from_xdcc_message(
                            f"/msg {start_bot} xdcc send #{start_pack + 1}-{end_pack}",
                            os.getcwd(),
                        )
                        combined = [manual] + from_message
                        # Start download
                        download_packs(combined)
                        Globalfunctions().retry()
                    else:
                        print("Make sure to choose from single bot")
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")


############################################################################



"""Manga Sources"""

# Importing necessary modules
import os
import re
import subprocess
import time

import requests
from bs4 import BeautifulSoup
from downitall.resources.globalfunctions import Globalfunctions, Manga
from selenium.common.exceptions import (
    NoSuchElementException,
    SessionNotCreatedException,
    TimeoutException,
    WebDriverException,
)
from selenium.webdriver.common.by import By

driver = Globalfunctions().webdriver()[0]
bookcli = Globalfunctions().webdriver()[1]


class Mangasources:
    ############################################################################
    ############################################################################
    class Bato:
        def bato_search(self, search_term):
            Globalfunctions().directory(name="Manga")
            # Declaring function level variables.
            label = "paged"
            index = 1
            # Making search term better for url through regex.
            term = re.sub(r"\W", "+", search_term)
            # Url to access webpage.
            web_url = f"https://bato.to/search?word={term}"
            # Url to be used for page navigation.
            page_url = f"https://bato.to/search?word={term}&page="
            # Url to access the base website
            base_url = "https://bato.to"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Checking weather the element exists.
                    if Globalfunctions().exists(
                        element=".pagination.pagination-sm.mb-0"
                    ):
                        html_tag = soup.find(
                            "ul", class_="pagination pagination-sm mb-0"
                        )
                        # Finding the last td tag and getting its text which is the number of last page and converting it into integer.
                        pages = int(
                            re.sub(r"\W+", "", html_tag.find_all("li")[-2].getText())
                        )
                    else:
                        pass
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all("a", class_="item-title", href=True)
                    index_list = []
                    name_list = []
                    link_list = []
                    img_list = []
                    # Finding the links and making link list.
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    index_list.append(0)
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        name_list.append(links.text.strip())
                        link_list.append(base_url + links["href"])
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    index_list.append(i + 1)
                    html_tag = soup.find_all("img", class_="rounded")
                    for img in html_tag:
                        img_list.append(img["src"])
                    while True:
                        print("\n")
                        for i, name in zip(index_list, name_list):
                            print(f"{i}. {name}")
                        # Calling the user choice function.
                        data = Globalfunctions().user_choice(
                            name_list, link_list, index_list, label
                        )
                        url = data[0]
                        name = data[1]
                        selection = data[2]
                        if name != "Next Page" and name != "Previous Page":
                            answer = Globalfunctions().confirmation(
                                selection, index_list, img_list
                            )
                            if answer == True:
                                break
                            else:
                                pass
                        else:
                            break
                    index = Globalfunctions().page_navigation(name, pages, index)
                    web_url = url + str(index)
                    if name != "Next Page" and name != "Previous Page":
                        break
                # Sending request to the webpage.
                label = "no_page"
                driver.get(url)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements in the webpage.
                html_tag = soup.find_all("a", class_="visited chapt", href=True)
                index_list = []
                name_list = []
                link_list = []
                # Finding the links and making link list.
                for i, links in enumerate(html_tag, start=1):
                    index_list.append(i)
                    name_list.append(links.text.strip())
                    link_list.append(base_url + links["href"])
                name_list.reverse()
                link_list.reverse()
                print("\n")
                for i, namess in zip(index_list, name_list):
                    print(f"{i}. {namess}")
                # Using core method as function to get rid of repeating the same lines.
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                url = data[0]
                chapter_name = data[1]
                # Sending request with selenium webdriver.
                driver.get(url)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                html_tag = soup.find_all("img", class_="page-img")
                # Creating the list to store image url.
                img_links_list = []
                # Finding all the mentioned elements from img_html_chapter.
                for imgs in html_tag:
                    img_links_list.append(imgs["src"])
                # Calling download_compress method as function.
                Manga().download_compress(name, chapter_name, img_links_list)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Weebcentral:
        def weebcentral(self, search_term):
            Globalfunctions().directory(name="Manga")
            # Declaring function level variables.
            label = "next_only"
            # Making search term better for url through regex.
            term = re.sub(r"\W", "+", search_term)
            # Url to access webpage.
            web_url = f"https://weebcentral.com/search?text={term}&sort=Best+Match&order=Descending&official=Any&anime=Any&adult=Any&display_mode=Full+Display"
            # Url to access the base website
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                time.sleep(2)
                while True:
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all("a", class_="line-clamp-1 link link-hover")
                    index_list = []
                    name_list = []
                    link_list = []
                    img_list = []
                    # Finding the links and making link list.
                    name_list.append("Next Page")
                    link_list.append("")
                    index_list.append(0)
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        name_list.append(links.text.strip())
                        link_list.append(links["href"])
                    html_tag = soup.find_all(
                        "img", {"width": "400", "height": "600", "decoding": "async"}
                    )
                    for img in html_tag:
                        img_list.append(img["src"])
                    print("\n")
                    for i, name in zip(index_list, name_list):
                        print(f"{i}. {name}")
                    # Calling the user choice function.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if name == "Next Page":
                        try:
                            driver.find_element(
                                By.XPATH,
                                '//button[contains(@class, "btn") and contains(@class, "bg-base-300")]',
                            ).click()
                            time.sleep(3)
                        except NoSuchElementException:
                            pass
                    else:
                        break
                # Sending request to the webpage.
                label = "no_page"
                driver.get(url)
                time.sleep(5)
                try:
                    driver.find_element(
                        By.XPATH,
                        "//button[@class='hover:bg-base-300 p-2' and @hx-target='#chapter-list']",
                    ).click()
                except NoSuchElementException:
                    pass
                time.sleep(2)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements in the webpage.
                html_tag = soup.find_all(
                    "a",
                    class_="hover:bg-base-300 flex-1 flex items-center p-2",
                    href=True,
                )
                html_tag_name = soup.find_all(
                    lambda tag: tag.name == "span"
                    and tag.has_attr("class")
                    and tag["class"] == []
                )
                index_list = []
                name_list = []
                link_list = []
                # Finding the links and making link list.
                for i, links in enumerate(html_tag, start=1):
                    index_list.append(i)
                    link_list.append(links["href"])
                for names in html_tag_name:
                    name_list.append(names.text)
                name_list.reverse()
                link_list.reverse()
                print("\n")
                for i, names in zip(index_list, name_list):
                    print(f"{i}. {names}")
                # Using core method as function to get rid of repeating the same lines.
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                url = data[0]
                chapter_name = data[1]
                # Sending request with selenium webdriver.
                driver.get(url)
                time.sleep(5)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                html_tag = soup.find_all("img", class_="maw-w-full mx-auto")
                # Creating the list to store image url.
                img_links_list = []
                # Finding all the mentioned elements from img_html_chapter.
                for imgs in html_tag:
                    img_links_list.append(imgs["src"])
                # Calling download_compress method as function.
                Manga().download_compress(name, chapter_name, img_links_list)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

        ############################################################################
        def weebcentral_batch(self, search_term):
            Globalfunctions().directory(name="Manga")
            # Declaring function level variables.
            label = "next_only"
            # Making search term better for url through regex.
            term = re.sub(r"\W", "+", search_term)
            # Url to access webpage.
            web_url = f"https://weebcentral.com/search?text={term}&sort=Best+Match&order=Descending&official=Any&anime=Any&adult=Any&display_mode=Full+Display"
            # Url to access the base website
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                time.sleep(2)
                while True:
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all("a", class_="line-clamp-1 link link-hover")
                    index_list = []
                    name_list = []
                    link_list = []
                    # Finding the links and making link list.
                    name_list.append("Next Page")
                    link_list.append("")
                    index_list.append(0)
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        name_list.append(links.text.strip())
                        link_list.append(links["href"])
                    print("\n")
                    for i, name in zip(index_list, name_list):
                        print(f"{i}. {name}")
                    # Calling the user choice function.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if name == "Next Page":
                        try:
                            driver.find_element(
                                By.XPATH,
                                '//button[contains(@class, "btn") and contains(@class, "bg-base-300")]',
                            ).click()
                            time.sleep(3)
                        except NoSuchElementException:
                            pass
                    else:
                        break
                # Sending request to the webpage.
                label = "no_page"
                driver.get(url)
                time.sleep(5)
                try:
                    driver.find_element(
                        By.XPATH,
                        "//button[@class='hover:bg-base-300 p-2' and @hx-target='#chapter-list']",
                    ).click()
                except NoSuchElementException:
                    pass
                time.sleep(2)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements in the webpage.
                html_tag = soup.find_all(
                    "a",
                    class_="hover:bg-base-300 flex-1 flex items-center p-2",
                    href=True,
                )
                html_tag_name = soup.find_all(
                    lambda tag: tag.name == "span"
                    and tag.has_attr("class")
                    and tag["class"] == []
                )
                name_list = []
                link_list = []
                # Finding the links and making link list.
                for links in html_tag:
                    link_list.append(links["href"])
                for names in html_tag_name:
                    name_list.append(names.text)
                name_list.reverse()
                link_list.reverse()
                Manga.batch_download(name_list, link_list, name)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except requests.exceptions.RequestException as e:
                print(e)
            except WebDriverException as e:
                print(e)
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Nyaa:
        def nyaa_search(self, search_term):
            Globalfunctions().directory(name="Manga")
            label = "paged"
            index = 1
            term = re.sub(r"\W", "+", search_term)
            # Url to access the searching.
            web_url = f"https://nyaa.si/?q={term}&f=0&c=3_1&s=seeders&o=desc"
            page_url = f"https://nyaa.si/?q={term}&f=0&c=3_1&s=seeders&o=desc&p="
            # Url to access the base website
            base_url = "https://nyaa.si"
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    if Globalfunctions().exists(element=".pagination"):
                        html_tag = soup.find("ul", class_="pagination")
                        pages = int(
                            html_tag.find_all("li", class_=None)[-1].getText().strip()
                        )
                    else:
                        pages = 1
                    # Finding all the mentioned elements from webpage.
                    table = soup.find(
                        "table",
                        class_="table table-bordered table-hover table-striped torrent-list",
                    )
                    df = Globalfunctions().visual_tables(table)
                    df_table = df.iloc[:, [1, 3]]
                    df_table.index += 1
                    index_list = df_table.index.tolist()
                    name_list = df[df.columns[1]].values.tolist()
                    link_list = []
                    td_tag = soup.find_all("td", colspan="2")
                    for a_html in td_tag:
                        for links in a_html.find_all(
                            "a", title=True, class_=None, href=True
                        ):
                            link_list.append(base_url + links["href"])
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    index_list.append(len(df_table.index.tolist()) + 1)
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    print("\n0. Next Page")
                    print(df_table)
                    print(f"{len(df_table.index.tolist())+1}. Previous Page")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    index = Globalfunctions().page_navigation(name, pages, index)
                    web_url = url + str(index)
                    if name != "Next Page" and name != "Previous Page":
                        break
                driver.get(url)
                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the "a" elements in the webpage.
                html_tag = soup.find("a", class_="card-footer-item")
                driver.quit()
                dir = os.getcwd()
                url = html_tag["href"]
                args = [
                    "aria2c",
                    "--file-allocation=none",
                    "--seed-time=0",
                    "-d",
                    dir,
                    url,
                ]
                subprocess.call(args)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Rawotaku:
        def rawotaku(self, search_term):
            Globalfunctions().directory(name="Manga")
            # Declaring function level variables.
            label = "no_page"
            # Making search term better for url through regex.
            term = re.sub(r"\W", "+", search_term)
            # Url to access webpage.
            web_url = f"https://rawotaku.com/?q={term}"
            # Url to be used for page navigation.
            try:
                while True:
                    # Sending request to the webpage.
                    driver.get(web_url)
                    time.sleep(5)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Checking weather the element exists.
                    index_list = []
                    name_list = []
                    link_list = []
                    # Finding the links and making link list.
                    html_tag = soup.find_all("a", class_="manga-poster")
                    html_tag_name = soup.find_all(
                        "img", class_="manga-poster-img lazyloaded"
                    )
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        link_list.append(links["href"])
                    for names in html_tag_name:
                        name_list.append(names["alt"].strip())
                    # Calling the user choice function.
                    print("\n")
                    for i, name in zip(index_list, name_list):
                        print(f"{i}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    break
                driver.get(url)
                time.sleep(5)
                # Getting html page with BeautifulSoup module
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Checking weather the element exists.
                index_list = []
                name_list = []
                link_list = []
                # Finding the links and making link list.
                html_tag = soup.find_all("a", class_="item-link")
                for i, links in enumerate(html_tag, start=1):
                    index_list.append(i)
                    link_list.append(links["href"])
                    name_list.append(links["title"].strip())
                link_list.reverse()
                name_list.reverse()
                # Calling the user choice function.
                print("\n")
                for i, namess in zip(index_list, name_list):
                    print(f"{i}. {namess}")
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                url = data[0]
                chapter_name = data[1]
                driver.get(url)
                last_height = driver.execute_script("return document.body.scrollHeight")
                current_position = 0

                while current_position < last_height:
                    current_position += 300
                    driver.execute_script(f"window.scrollTo(0, {current_position});")
                    time.sleep(0.2)
                    last_height = driver.execute_script(
                        "return document.body.scrollHeight"
                    )

                # Parsering the response with "BeauitifulSoup".
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                html_tag = soup.find_all("img", class_="image-vertical lazyloaded")
                # Creating the list to store image url.
                img_links_list = []
                # Finding all the mentioned elements from img_html_chapter.
                for imgs in html_tag:
                    img_links_list.append(imgs["data-src"])
                # Calling download_compress method as function.
                Manga().download_compress(name, chapter_name, img_links_list)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Readallcomics:
        def readallcomics(self, search_term):
            Globalfunctions().directory(name="Manga")
            # Declaring function level variables.
            label = "no_page"
            # Url to access webpage.
            web_url = f"https://readallcomics.com/?story={search_term}&s=&type=comic"
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                while True:
                    time.sleep(5)
                    # Getting html page with BeautifulSoup module
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all("a", href=True, title=True)
                    index_list = []
                    name_list = []
                    link_list = []
                    # Finding the links and making link list.
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        name_list.append(links["title"])
                        link_list.append(links["href"])
                    print("\n")
                    for i, name in zip(index_list, name_list):
                        print(f"{i}. {name}")
                    # Calling the user choice function.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    break
                # Sending request to the webpage.
                driver.get(url)
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Finding all the mentioned elements from webpage.
                html_tag = soup.find_all("a", href=True, title=True)
                index_list = []
                name_list = []
                link_list = []
                # Finding the links and making link list.
                for i, links in enumerate(html_tag, start=1):
                    index_list.append(i)
                    name_list.append(links["title"])
                    link_list.append(links["href"])
                link_list.reverse()
                name_list.reverse()
                print("\n")
                for i, namess in zip(index_list, name_list):
                    print(f"{i}. {namess}")
                # Calling the user choice function.
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                url = data[0]
                chapter_name = data[1]
                driver.get(url)
                print("Waiting for pages to load...")
                last_height = driver.execute_script("return document.body.scrollHeight")
                for y in range(0, last_height, 300):
                    driver.execute_script(f"window.scrollTo(0, {y});")
                    time.sleep(0.2)
                driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
                soup = BeautifulSoup(driver.page_source, "html.parser")
                html_tag = soup.find_all(
                    "img", class_="bgimglazy-load", attrs={"decoding": "async"}
                )
                # Creating the list to store image url.
                img_links_list = []
                # Finding all the mentioned elements from img_html_chapter.
                for imgs in html_tag:
                    img_links_list.append(imgs["src"])
                # Calling download_compress method as function.
                Manga().download_compress(name, chapter_name, img_links_list)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")


############################################################################



"""Stream Sources"""

# Importing necessary modules
import re
import time
import urllib.parse

import requests
from bs4 import BeautifulSoup
from ipytv import playlist
from selenium.common.exceptions import (
    SessionNotCreatedException,
    TimeoutException,
    WebDriverException,
)
from selenium.webdriver.common.by import By
from selenium.webdriver.support.select import Select

from downitall.resources.globalfunctions import Globalfunctions, Stream

driver = Globalfunctions().webdriver()[0]
bookcli = Globalfunctions().webdriver()[1]


class Streamsources:
    ############################################################################
    ############################################################################
    class Anizone:
        def anizone(self, search_term):
            # Declaring function level variables.
            label = "no_page"
            # Url to access webpage.
            url = f"https://anizone.to/anime?search={search_term}"
            try:
                # Sending request to the webpage.
                driver.get(url)
                # Getting html page with BeautifulSoup module
                time.sleep(5)
                soup = BeautifulSoup(driver.page_source, "html.parser")
                # Checking weather the element exists.
                # Finding all the mentioned elements from webpage.
                html_tag = soup.find_all(
                    "a",
                    class_="text-white inline hocus:text-amber-300 transition-colors duration-300 ease-in-out focus:outline-none focus:ring-0",
                )
                index_list = []
                name_list = []
                link_list = []
                # Finding the links and making link list.
                for i, links in enumerate(html_tag, start=1):
                    index_list.append(i)
                    link_list.append(links["href"])
                    name_list.append(links.get("title"))
                print("\n")
                for index_name, name in zip(index_list, name_list):
                    # Printing name with index for user to choose.
                    print(f"{index_name}. {name}")
                # Calling the user choice function.
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                url = data[0]
                driver.get(url)
                time.sleep(3)
                soup = BeautifulSoup(driver.page_source, "html.parser")
                html_tag = soup.find_all(
                    "a",
                    class_="group block relative drop-shadow-lg overflow-hidden h-[101px] px-5 py-4 bg-slate-900 rounded-lg focus:outline-none",
                )
                html_tag_name = soup.find_all(
                    "h3",
                    class_="group-hocus:text-amber-300 transition-colors duration-300 ease-in-out",
                )
                index_list = []
                name_list = []
                link_list = []
                # Finding the links and making link list.
                for i, links in enumerate(html_tag, start=1):
                    index_list.append(i)
                    link_list.append(links["href"])
                for names in html_tag_name:
                    name_list.append(names.text.strip())
                # Calling the user choice function.
                print("\n")
                for index_name, name in zip(index_list, name_list):
                    # Printing name with index for user to choose.
                    print(f"{index_name}. {name}")
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                url = data[0]
                driver.get(url)
                time.sleep(3)
                soup = BeautifulSoup(driver.page_source, "html.parser")
                html_tag = soup.find_all("source")[0]
                url = html_tag["src"]
                Stream().link_check(url)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Miruro:
        def miruro(self, search_term):
            # Declaring function level variables.
            label = "next_only"
            # Url to access webpage.
            web_url = f"https://www.miruro.tv/search?query={search_term}&sort=POPULARITY_DESC&type=ANIME"
            base_url = "https://www.miruro.tv"
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                print("Wait a few moments")
                time.sleep(6)
                while True:
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Checking weather the element exists.
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all(
                        "a", title=True, href=lambda t: t and "watch" in t
                    )
                    index_list = []
                    name_list = []
                    link_list = []
                    # Finding the links and making link list.
                    index_list.append(0)
                    name_list.append("Next Page")
                    link_list.append("")
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        name_list.append(links["title"].strip())
                        link_list.append(base_url + links["href"])
                    # Calling the user choice function.
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    if name == "Next Page":
                        scroll_amount = 1000
                        driver.execute_script(f"window.scrollBy(0, {scroll_amount})")
                        time.sleep(2)
                    else:
                        break
                driver.get(url)
                time.sleep(10)
                soup = BeautifulSoup(driver.page_source, "html.parser")
                select_list = []
                select_name = []
                html_tag = soup.find_all("select")[0]
                tag = html_tag.find_all("option")
                for values in tag:
                    select_list.append(values["value"])
                    select_name.append(values.text.strip())
                for i, names in enumerate(select_name, start=1):
                    print(f"{i}. {names}")
                user_input = input("Select an index: ")
                select = Select(driver.find_element(By.TAG_NAME, "select"))
                select.select_by_value(f"{select_list[int(user_input)-1]}")
                time.sleep(5)
                soup = BeautifulSoup(driver.page_source, "html.parser")
                html_tag = soup.find_all("div", id="episodes-list-container")[0]
                html_tag_name = html_tag.find_all("button")
                index_list = []
                name_list = []
                link_list = []
                # Finding the links and making link list.
                label = "no_page"
                for i, names in enumerate(html_tag_name, start=1):
                    index_list.append(i)
                    name_list.append(names["title"].strip())
                    for links in names.find_all("a"):
                        link_list.append(base_url + links["href"])
                # Calling the user choice function.
                print("\n")
                for index_name, name in zip(index_list, name_list):
                    # Printing name with index for user to choose.
                    print(f"{index_name}. {name}")
                data = Globalfunctions().user_choice(
                    name_list, link_list, index_list, label
                )
                url = data[0]
                driver.get(url)
                time.sleep(5)
                soup = BeautifulSoup(driver.page_source, "html.parser")
                html_tag = soup.find_all("source")[0]
                proxy_url = html_tag["src"]
                # Parse the full URL
                parsed_url = urllib.parse.urlparse(proxy_url)

                # Extract the 'url' query parameter
                query_params = urllib.parse.parse_qs(parsed_url.query)
                encoded_m3u8_url = query_params.get("url", [None])[0]

                # Decode the percent-encoded string
                url = urllib.parse.unquote(encoded_m3u8_url)

                Stream().link_check(url)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Hydrahd:
        def hydrahd(self, search_term):
            # Declaring function level variables.
            label = "paged"
            index = 1
            # Url to access webpage.
            web_url = f"https://hydrahd.sh/index.php?menu=search&query={search_term}"
            base_url = "https://hydrahd.sh"
            page_url = f"https://hydrahd.sh/search/{search_term}/"
            try:
                # Sending request to the webpage.
                while True:
                    driver.get(web_url)
                    time.sleep(3)
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Checking weather the element exists.
                    if Globalfunctions().exists(element="pagination"):
                        html_tag = soup.find("ul", class_="pagination")
                        # Finding the last td tag and getting its text which is the number of last page and converting it into integer.
                        pages = 6
                    else:
                        pages = 6
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all("a", class_="hthis")
                    index_list = []
                    name_list = []
                    link_list = []
                    # Finding the links and making link list.
                    name_list.append("Next Page")
                    link_list.append(page_url)
                    index_list.append(0)
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        link_list.append(base_url + links["href"])
                        name_list.append(links["title"].strip())
                    name_list.append("Previous Page")
                    link_list.append(page_url)
                    index_list.append(i + 1)
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    # Calling the user choice function.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    index = Globalfunctions().page_navigation(name, pages, index)
                    web_url = url + str(index)
                    if name != "Next Page" and name != "Previous Page":
                        break
                if "watchseries" in url:
                    driver.get(url)
                    time.sleep(2)
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    html_tag = soup.find_all(
                        "a", class_="dynamic-ep-link visitedep link episodeColorBlack"
                    )
                    index_list = []
                    name_list = []
                    link_list = []
                    # Finding the links and making link list.
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        link_list.append(base_url + links["href"])
                        name_list.append(links["title"])
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                driver.get(url)
                time.sleep(4)
                play_button = driver.find_element(By.CLASS_NAME, "video-play-button")
                driver.execute_script("arguments[0].click();", play_button)
                print("Wait a few moments")
                time.sleep(15)
                stream_urls = []
                for request in driver.requests:
                    if request.response:
                        url = request.url
                        if ".m3u8" in url or ".mp4" in url or ".ts" in url:
                            stream_urls.append(url)
                url = stream_urls[0]
                Stream().link_check(url)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except requests.exceptions.RequestException as e:
                print(e)
            except WebDriverException as e:
                print(e)
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Cxtvlive:
        def cxtvlive(self):
            # Declaring function level variables.
            label = "no_page"
            # Url to access webpage.
            web_url = "https://www.cxtvlive.com/tv/top"
            try:
                # Sending request to the webpage.
                driver.get(web_url)
                time.sleep(5)
                while True:
                    soup = BeautifulSoup(driver.page_source, "html.parser")
                    # Checking weather the element exists.
                    # Finding all the mentioned elements from webpage.
                    html_tag = soup.find_all(
                        "a", class_="m-b-20 btn btn-white btn-block"
                    )
                    html_tag_name = soup.find_all(
                        "h4",
                        class_="resumo p-t-15 p-b-5 no-margin text-center bold",
                    )
                    index_list = []
                    name_list = []
                    link_list = []
                    # Finding the links and making link list.
                    for i, links in enumerate(html_tag, start=1):
                        index_list.append(i)
                        link_list.append(links["href"])
                    for names in html_tag_name:
                        name_list.append(names.text.strip())
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    # Calling the user choice function.
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    name = data[1]
                    break
                driver.get(url)
                time.sleep(2)
                soup = BeautifulSoup(driver.page_source, "html.parser")
                html_tag = soup.find_all("iframe")[0]
                if not "youtube" in html_tag["src"]:
                    html_tag = soup.find_all("source")[0]
                    url = re.sub("&amp;+", "", html_tag["src"])
                elif not "list" in html_tag["src"] and "youtube" in html_tag["src"]:
                    url = html_tag["src"]
                else:
                    raise IndexError
                Stream().link_check(url)
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except requests.exceptions.RequestException as e:
                print(e)
            except WebDriverException as e:
                print(e)
            except TypeError as e:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")

    ############################################################################
    ############################################################################
    class Iptv:
        def iptv(self, choice):
            # Declaring function level variables.
            label = "no_page"
            # Url to access webpage.
            if choice == 1:
                web_url = "https://iptv-org.github.io/iptv/languages/eng.m3u"
            else:
                web_url = "https://raw.githubusercontent.com/FunctionError/PiratesTv/main/combined_playlist.m3u"
            try:
                pl = playlist.loadu(web_url)
                # Let's retrieve the first channel in the list
                index_list = []
                name_list = []
                link_list = []
                for i, channel in enumerate(pl, start=1):
                    index_list.append(i)
                    if (
                        not "[Not 24/7]" in channel.name
                        and not "[Geo-blocked]" in channel.name
                    ):
                        name_list.append(channel.name.strip())
                    if "m3u8" in channel.url or "mpd" in channel.url:
                        link_list.append(channel.url)
                while True:
                    print("\n")
                    for index_name, name in zip(index_list, name_list):
                        # Printing name with index for user to choose.
                        print(f"{index_name}. {name}")
                    data = Globalfunctions().user_choice(
                        name_list, link_list, index_list, label
                    )
                    url = data[0]
                    if Stream().link_check(url) == True:
                        break
                Globalfunctions().retry()
            except SessionNotCreatedException:
                print(
                    "\nIf you are not using android then install from win_linux_requirement.txt file"
                )
            except (
                requests.exceptions.RequestException,
                WebDriverException,
                TimeoutException,
            ):
                print("\nNetwork Error!")
            except TypeError:
                pass
            except (IndexError, AttributeError, UnboundLocalError):
                print("\nCould not find any thing :(")
            except KeyboardInterrupt:
                print("\n\nCancelled by user.")



"""Setup"""

import os
import subprocess
from pathlib import Path

from setuptools import setup

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

# Set a reasonable umask.
os.umask(0o022)
# Make all local files readable and all directories listable.
subprocess.call(["chmod", "-R", "a+rX", "."])

setup(
    name="downitall_windows",
    version="1.5",
    description="A cli to download and stream all kind of media",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Behram Akhtar/Aatiho Nora",
    author_email="behramakhtar@gmail.com",
    packages=["downitall", "downitall.resources"],
    package_data={"": ["license.txt"]},
    include_package_data=True,
    install_requires=[
        "requests==2.32.4",
        "bs4==0.0.2",
        "selenium==4.9.1",
        "selenium-wire==5.1.0",
        "blinker==1.7.0",
        "cbz-generator==1.0.0",
        "tqdm==4.67.1",
        "gdown==5.2.0",
        "term-image==0.7.2",
        "questionary==2.1.0",
        "ytmusicapi==1.10.3",
        "xdcc-dl==5.2.1",
        "streamlink==7.5.0",
        "m3u-ipytv==0.2.11",
        "setuptools==80.9.0",
        "yt-dlp==2025.6.30",
    ],  # external packages as dependencies
    scripts=["bin/downitall"],
)
